@@ -4,6 +4,7 @@ Tests for the core logic.
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
+from __future__ import print_function
 import copy
 import logging
 import os
@@ -12,11 +13,11 @@ import time
 import uuid
 import unittest
 import warnings
-import Pyro4.configuration
 import Pyro4.core
 import Pyro4.errors
 import Pyro4.constants
 import Pyro4.futures
+from Pyro4.configuration import config
 from testsupport import *
 
 
@@ -38,35 +39,35 @@ class CoreTests(unittest.TestCase):
 
     def testProxyNoHmac(self):
         # check that proxy without hmac is possible
-        with Pyro4.Proxy("PYRO:object@host:9999") as p:
+        with Pyro4.core.Proxy("PYRO:object@host:9999") as p:
             pass
 
     def testDaemonNoHmac(self):
         # check that daemon without hmac is possible
-        d = Pyro4.Daemon()
+        d = Pyro4.core.Daemon()
         d.shutdown()
 
     def testConfig(self):
-        self.assertTrue(type(Pyro4.config.COMPRESSION) is bool)
-        self.assertTrue(type(Pyro4.config.NS_PORT) is int)
-        config = Pyro4.config.asDict()
-        self.assertTrue(type(config) is dict)
-        self.assertIn("COMPRESSION", config)
-        self.assertEqual(Pyro4.config.COMPRESSION, config["COMPRESSION"])
+        self.assertTrue(type(config.COMPRESSION) is bool)
+        self.assertTrue(type(config.NS_PORT) is int)
+        cfgdict = config.asDict()
+        self.assertTrue(type(cfgdict) is dict)
+        self.assertIn("COMPRESSION", cfgdict)
+        self.assertEqual(config.COMPRESSION, cfgdict["COMPRESSION"])
 
     def testConfigDefaults(self):
         # some security sensitive settings:
-        Pyro4.config.reset(False)   # reset the config to default
-        self.assertTrue(Pyro4.config.REQUIRE_EXPOSE)
-        self.assertEqual("localhost", Pyro4.config.HOST)
-        self.assertEqual("localhost", Pyro4.config.NS_HOST)
-        self.assertFalse(Pyro4.config.FLAME_ENABLED)
-        self.assertEqual("serpent", Pyro4.config.SERIALIZER)
-        self.assertEqual({"json", "serpent", "marshal"}, Pyro4.config.SERIALIZERS_ACCEPTED)
+        config.reset(False)   # reset the config to default
+        self.assertTrue(config.REQUIRE_EXPOSE)
+        self.assertEqual("localhost", config.HOST)
+        self.assertEqual("localhost", config.NS_HOST)
+        self.assertFalse(config.FLAME_ENABLED)
+        self.assertEqual("serpent", config.SERIALIZER)
+        self.assertEqual({"json", "serpent", "marshal"}, config.SERIALIZERS_ACCEPTED)
 
     def testConfigValid(self):
         try:
-            Pyro4.config.XYZ_FOOBAR = True  # don't want to allow weird config names
+            config.XYZ_FOOBAR = True  # don't want to allow weird config names
             self.fail("expected exception for weird config item")
         except AttributeError:
             pass
@@ -111,7 +112,7 @@ class CoreTests(unittest.TestCase):
         self.assertEqual(uri, str(p))
         uri = "PYRONAME:some_obj_name@host.com"
         p = Pyro4.core.URI(uri)
-        self.assertEqual(uri + ":" + str(Pyro4.config.NS_PORT), str(p))  # a PYRONAME uri with a hostname gets a port too if omitted
+        self.assertEqual(uri + ":" + str(config.NS_PORT), str(p))  # a PYRONAME uri with a hostname gets a port too if omitted
         uri = "PYRONAME:some_obj_name@host.com:8888"
         p = Pyro4.core.URI(uri)
         self.assertEqual(uri, str(p))
@@ -166,12 +167,17 @@ class CoreTests(unittest.TestCase):
         self.assertEqual("host.com", p.host)
         self.assertIsNone(p.sockname)
         self.assertEqual(4444, p.port)
+
+    def testUriParsingIpv6(self):
         p = Pyro4.core.URI("pyro:12345@[::1]:4444")
         self.assertEqual("::1", p.host)
         self.assertEqual("[::1]:4444", p.location)
         with self.assertRaises(Pyro4.errors.PyroError) as e:
             Pyro4.core.URI("pyro:12345@[[::1]]:4444")
         self.assertEqual("invalid ipv6 address: enclosed in too many brackets", str(e.exception))
+        with self.assertRaises(Pyro4.errors.PyroError) as e:
+            Pyro4.core.URI("pyro:12345@[must_be_numeric_here]:4444")
+        self.assertEqual("invalid ipv6 address: the part between brackets must be a numeric ipv6 address", str(e.exception))
 
     def testUriParsingPyroname(self):
         p = Pyro4.core.URI("PYRONAME:objectname")
@@ -183,7 +189,7 @@ class CoreTests(unittest.TestCase):
         self.assertEqual("PYRONAME", p.protocol)
         self.assertEqual("objectname", p.object)
         self.assertEqual("nameserverhost", p.host)
-        self.assertEqual(Pyro4.config.NS_PORT, p.port)  # Pyroname uri with host gets a port too if not specified
+        self.assertEqual(config.NS_PORT, p.port)  # Pyroname uri with host gets a port too if not specified
         p = Pyro4.core.URI("PYRONAME:objectname@nameserverhost:4444")
         self.assertEqual("PYRONAME", p.protocol)
         self.assertEqual("objectname", p.object)
@@ -204,7 +210,7 @@ class CoreTests(unittest.TestCase):
         self.assertEqual("PYROMETA", p.protocol)
         self.assertEqual({"meta1", "meta2"}, p.object)
         self.assertEqual("nameserverhost", p.host)
-        self.assertEqual(Pyro4.config.NS_PORT, p.port)  # PyroMeta uri with host gets a port too if not specified
+        self.assertEqual(config.NS_PORT, p.port)  # PyroMeta uri with host gets a port too if not specified
         p = Pyro4.core.URI("PYROMETA:meta@nameserverhost:4444")
         self.assertEqual("PYROMETA", p.protocol)
         self.assertEqual({"meta"}, p.object)
@@ -370,16 +376,16 @@ class CoreTests(unittest.TestCase):
         p._pyroRelease()
         p2._pyroRelease()
 
-    def testAsyncProxyAdapterCopy(self):
+    def testAsyncProxyCopy(self):
         try:
-            Pyro4.config.METADATA = False
+            config.METADATA = False
             with Pyro4.core.Proxy("PYRO:12345@hostname:9999") as proxy:
-                asyncproxy = proxy._pyroAsync()
-                p2 = copy.copy(asyncproxy)
+                proxy._pyroAsync()
+                p2 = copy.copy(proxy)
                 asynccall = p2.foobar()
                 self.assertIsInstance(asynccall, Pyro4.futures.FutureResult)
         finally:
-            Pyro4.config.METADATA = True
+            config.METADATA = True
 
     def testBatchProxyAdapterCopy(self):
         with Pyro4.core.Proxy("PYRO:12345@hostname:9999") as proxy:
@@ -413,16 +419,16 @@ class CoreTests(unittest.TestCase):
             self.assertEqual(unicode(expected), unicode(p))
 
     def testProxySerializerOverride(self):
-        serializer = Pyro4.config.SERIALIZER
+        serializer = config.SERIALIZER
         try:
-            Pyro4.config.SERIALIZER = "~invalid~"
+            config.SERIALIZER = "~invalid~"
             _ = Pyro4.core.Proxy("PYRO:obj@localhost:5555")
             self.fail("must raise exception")
         except Pyro4.errors.SerializeError as x:
             self.assertIn("~invalid~", str(x))
             self.assertIn("unknown", str(x))
         finally:
-            Pyro4.config.SERIALIZER = serializer
+            config.SERIALIZER = serializer
         try:
             proxy = Pyro4.core.Proxy("PYRO:obj@localhost:5555")
             proxy._pyroSerializer = "~invalidoverride~"
@@ -434,7 +440,7 @@ class CoreTests(unittest.TestCase):
             self.assertIn("unknown", str(x))
         finally:
             proxy._pyroConnection = None
-            Pyro4.config.SERIALIZER = serializer
+            config.SERIALIZER = serializer
 
     def testProxyDir(self):
         # PyPy tries to call deprecated __members__ and __methods__
@@ -566,12 +572,12 @@ class CoreTests(unittest.TestCase):
     def testTimeoutGetSet(self):
         class ConnectionMock(object):
             def __init__(self):
-                self.timeout = Pyro4.config.COMMTIMEOUT
+                self.timeout = config.COMMTIMEOUT
 
             def close(self):
                 pass
 
-        Pyro4.config.COMMTIMEOUT = None
+        config.COMMTIMEOUT = None
         p = Pyro4.core.Proxy("PYRO:obj@host:555")
         self.assertIsNone(p._pyroTimeout)
         p._pyroTimeout = 5
@@ -582,7 +588,7 @@ class CoreTests(unittest.TestCase):
         p._pyroTimeout = 5
         self.assertEqual(5, p._pyroTimeout)
         self.assertEqual(5, p._pyroConnection.timeout)
-        Pyro4.config.COMMTIMEOUT = 2
+        config.COMMTIMEOUT = 2
         p = Pyro4.core.Proxy("PYRO:obj@host:555")
         p._pyroConnection = ConnectionMock()
         self.assertEqual(2, p._pyroTimeout)
@@ -590,14 +596,14 @@ class CoreTests(unittest.TestCase):
         p._pyroTimeout = None
         self.assertIsNone(p._pyroTimeout)
         self.assertIsNone(p._pyroConnection.timeout)
-        Pyro4.config.COMMTIMEOUT = None
+        config.COMMTIMEOUT = None
         p._pyroRelease()
 
     def testCallbackDecorator(self):
         # just test the decorator itself, testing the callback
         # exception handling is kinda hard in unit tests. Maybe later.
         class Test(object):
-            @Pyro4.callback
+            @Pyro4.core.callback
             def method(self):
                 pass
 
@@ -635,8 +641,8 @@ class CoreTests(unittest.TestCase):
         corr_id2 = uuid.UUID('67b05ad9-2d6a-4ed8-8ed5-95cba68b4cf9')
         d["correlation_id"] = corr_id2
         ctx.from_global(d)
-        self.assertEqual(corr_id2, Pyro4.current_context.correlation_id)
-        Pyro4.current_context.correlation_id = None
+        self.assertEqual(corr_id2, Pyro4.core.current_context.correlation_id)
+        Pyro4.core.current_context.correlation_id = None
 
 
 class ExposeDecoratorTests(unittest.TestCase):
@@ -711,16 +717,16 @@ class BehaviorDecoratorTests(unittest.TestCase):
         self.assertIs(float, ic)
 
     def testBehaviorWithExposeKeepsCorrectValues(self):
-        @Pyro4.behavior(instance_mode="percall", instance_creator=float)
-        @Pyro4.expose
+        @Pyro4.core.behavior(instance_mode="percall", instance_creator=float)
+        @Pyro4.core.expose
         class TestClass:
             pass
         im, ic = TestClass._pyroInstancing
         self.assertEqual("percall", im)
         self.assertIs(float, ic)
 
-        @Pyro4.expose
-        @Pyro4.behavior(instance_mode="percall", instance_creator=float)
+        @Pyro4.core.expose
+        @Pyro4.core.behavior(instance_mode="percall", instance_creator=float)
         class TestClass2:
             pass
         im, ic = TestClass2._pyroInstancing
@@ -770,7 +776,7 @@ class RemoteMethodTests(unittest.TestCase):
         def __exit__(self, *args):
             pass
 
-        def _pyroAsync(self):
+        def _pyroAsync(self, async=True):
             return self
 
         def __getattr__(self, item):
@@ -785,7 +791,7 @@ class RemoteMethodTests(unittest.TestCase):
 
     def testRemoteMethodMetaOff(self):
         try:
-            Pyro4.config.METADATA = False
+            config.METADATA = False
             class ProxyMock(object):
                 def invoke(self, name, args, kwargs):
                     return "INVOKED name=%s args=%s kwargs=%s" % (name, args, kwargs)
@@ -804,7 +810,7 @@ class RemoteMethodTests(unittest.TestCase):
             self.assertIsInstance(a2, Pyro4.core._RemoteMethod, "nested attribute should just be another RemoteMethod")
             p._pyroRelease()
         finally:
-            Pyro4.config.METADATA = True
+            config.METADATA = True
 
     def testRemoteMethodMetaOn(self):
         p = Pyro4.core.Proxy("PYRO:obj@localhost:59999")
@@ -814,7 +820,7 @@ class RemoteMethodTests(unittest.TestCase):
 
     def testBatchMethod(self):
         proxy = self.BatchProxyMock()
-        batch = Pyro4.batch(proxy)
+        batch = Pyro4.core.batch(proxy)
         self.assertIsNone(batch.foo(42))
         self.assertIsNone(batch.bar("abc"))
         self.assertIsNone(batch.baz(42, "abc", arg=999))
@@ -834,7 +840,7 @@ class RemoteMethodTests(unittest.TestCase):
 
     def testBatchMethodOneway(self):
         proxy = self.BatchProxyMock()
-        batch = Pyro4.batch(proxy)
+        batch = Pyro4.core.batch(proxy)
         self.assertIsNone(batch.foo(42))
         self.assertIsNone(batch.bar("abc"))
         self.assertIsNone(batch.baz(42, "abc", arg=999))
@@ -847,7 +853,7 @@ class RemoteMethodTests(unittest.TestCase):
 
     def testBatchMethodAsync(self):
         proxy = self.BatchProxyMock()
-        batch = Pyro4.batch(proxy)
+        batch = Pyro4.core.batch(proxy)
         self.assertIsNone(batch.foo(42))
         self.assertIsNone(batch.bar("abc"))
         self.assertIsNone(batch.pause(0.5))  # pause shouldn't matter with async
@@ -870,7 +876,7 @@ class RemoteMethodTests(unittest.TestCase):
 
     def testBatchMethodReuse(self):
         proxy = self.BatchProxyMock()
-        batch = Pyro4.batch(proxy)
+        batch = Pyro4.core.batch(proxy)
         batch.foo(1)
         batch.foo(2)
         results = batch()
@@ -883,11 +889,30 @@ class RemoteMethodTests(unittest.TestCase):
         results = batch()
         self.assertEqual(0, len(list(results)))
 
+    def testChangedAsyncLegacyApi(self):
+        try:
+            Pyro4.config.METADATA = False
+            proxy = Pyro4.Proxy("PYRO:test@addr:5555")
+            self.assertIsNone(proxy._pyroAsync())
+            m = proxy.foo
+            self.assertIsInstance(m, Pyro4.core._AsyncRemoteMethod)
+            self.assertIsNone(proxy._pyroAsync(False))
+            m = proxy.foo
+            self.assertIsInstance(m, Pyro4.core._RemoteMethod)
+            self.assertIsNone(Pyro4.async(proxy))
+            m = proxy.foo
+            self.assertIsInstance(m, Pyro4.core._AsyncRemoteMethod)
+            self.assertIsNone(Pyro4.async(proxy, False))
+            m = proxy.foo
+            self.assertIsInstance(m, Pyro4.core._RemoteMethod)
+        finally:
+            Pyro4.config.METADATA = True
+
     def testAsyncMethod(self):
         proxy = self.AsyncProxyMock()
-        async = Pyro4.async(proxy)
+        Pyro4.core.async(proxy)
         begin = time.time()
-        result = async.pause_and_divide(0.2, 10, 2)  # returns immediately
+        result = proxy.pause_and_divide(0.2, 10, 2)  # returns immediately
         duration = time.time() - begin
         self.assertLess(duration, 0.1)
         self.assertFalse(result.ready)
@@ -904,8 +929,8 @@ class RemoteMethodTests(unittest.TestCase):
                 return value + amount
 
         proxy = self.AsyncProxyMock()
-        async = Pyro4.async(proxy)
-        result = async.pause_and_divide(0.2, 10, 2)  # returns immediately
+        Pyro4.core.async(proxy)
+        result = proxy.pause_and_divide(0.2, 10, 2)  # returns immediately
         holder = AsyncFunctionHolder()
         result.then(holder.asyncFunction, amount=2) \
             .then(holder.asyncFunction, amount=4) \
@@ -923,8 +948,8 @@ class RemoteMethodTests(unittest.TestCase):
             return 1 // 0  # crash
 
         proxy = self.AsyncProxyMock()
-        async = Pyro4.async(proxy)
-        result = async.pause_and_divide(0.2, 10, 2)  # returns immediately
+        Pyro4.core.async(proxy)
+        result = proxy.pause_and_divide(0.2, 10, 2)  # returns immediately
         result.then(crashingAsyncFunction).then(normalAsyncFunction, 2)
         try:
             _ = result.value
@@ -935,8 +960,8 @@ class RemoteMethodTests(unittest.TestCase):
 
     def testAsyncMethodTimeout(self):
         proxy = self.AsyncProxyMock()
-        async = Pyro4.async(proxy)
-        result = async.pause_and_divide(1, 10, 2)  # returns immediately
+        Pyro4.core.async(proxy)
+        result = proxy.pause_and_divide(1, 10, 2)  # returns immediately
         self.assertFalse(result.ready)
         self.assertFalse(result.wait(0.5))  # won't be ready after 0.5 sec
         self.assertTrue(result.wait(1))  # will be ready within 1 seconds more
@@ -992,14 +1017,14 @@ class FuturesErrorHandlerStorage(object):
 
 class TestFutures(unittest.TestCase):
     def testSimpleFuture(self):
-        f = Pyro4.Future(futurestestfunc)
+        f = Pyro4.futures.Future(futurestestfunc)
         r = f(4, 5)
         self.assertIsInstance(r, Pyro4.futures.FutureResult)
         value = r.value
         self.assertEqual(9, value)
 
     def testFutureChain(self):
-        f = Pyro4.Future(futurestestfunc) \
+        f = Pyro4.futures.Future(futurestestfunc) \
             .then(futurestestfunc, 6) \
             .then(futurestestfunc, 7) \
             .then(futurestestfunc, 8) \
@@ -1009,7 +1034,7 @@ class TestFutures(unittest.TestCase):
         self.assertEqual(4 + 5 + 6 + 7 + 8 + 9 + 10, value)
 
     def testCrashingChain(self):
-        f = Pyro4.Future(futurestestfunc) \
+        f = Pyro4.futures.Future(futurestestfunc) \
             .then(futurestestfunc, 6) \
             .then(crashingfuturestestfunc) \
             .then(futurestestfunc, 8)
@@ -1022,7 +1047,7 @@ class TestFutures(unittest.TestCase):
 
     def testErrorHandler(self):
         storage = FuturesErrorHandlerStorage()
-        f = Pyro4.Future(crashingfuturestestfunc) \
+        f = Pyro4.futures.Future(crashingfuturestestfunc) \
             .then(futurestestfunc, 5) \
             .iferror(storage.errorhandler) \
             .then(futurestestfunc, 6)
@@ -1035,7 +1060,7 @@ class TestFutures(unittest.TestCase):
         self.assertIsInstance(storage.error, ZeroDivisionError)
 
     def testFutureResultChainSlow(self):
-        f = Pyro4.Future(futurestestfunc)
+        f = Pyro4.futures.Future(futurestestfunc)
         result = f(4, 5)
         time.sleep(.02)
         result.then(futurestestfunc, 6)
@@ -1052,7 +1077,7 @@ class TestFutures(unittest.TestCase):
         self.assertEqual(4 + 5 + 6 + 7 + 8 + 9 + 10, value)
 
     def testFutureResultChain(self):
-        f = Pyro4.Future(futurestestfunc)
+        f = Pyro4.futures.Future(futurestestfunc)
         result = f(4, 5).then(futurestestfunc, 6).then(futurestestfunc, 7).then(futurestestfunc, 8).then(futurestestfunc, 9).then(futurestestfunc, 10)
         value = result.value
         self.assertEqual(4 + 5 + 6 + 7 + 8 + 9 + 10, value)
@@ -1060,14 +1085,14 @@ class TestFutures(unittest.TestCase):
             f(4, 5)   # cannot evaluate the same future more than once
 
     def testFutureDelay(self):
-        f = Pyro4.Future(futurestestfunc)
+        f = Pyro4.futures.Future(futurestestfunc)
         b = f.delay(0)
         self.assertTrue(b)
         begin = time.time()
         f(4, 5).value
         duration = time.time() - begin
         self.assertLess(duration, 0.1)
-        f = Pyro4.Future(futurestestfunc)
+        f = Pyro4.futures.Future(futurestestfunc)
         b = f.delay(1)
         self.assertTrue(b)
         begin = time.time()
@@ -1077,19 +1102,19 @@ class TestFutures(unittest.TestCase):
         begin = time.time()
         r.value
         duration = time.time() - begin
-        self.assertGreaterEqual(duration, 1)
+        self.assertGreaterEqual(duration, 0.98)
         self.assertLess(duration, 1.1)
         self.assertFalse(f.delay(10))
 
     def testFutureCancel(self):
-        f = Pyro4.Future(futurestestfunc)
+        f = Pyro4.futures.Future(futurestestfunc)
         f.delay(10)
         b = f.cancel()
         self.assertTrue(b)
         with self.assertRaises(RuntimeError) as x:
             f(4, 5)
         self.assertTrue("cancelled" in str(x.exception))
-        f = Pyro4.Future(futurestestfunc)
+        f = Pyro4.futures.Future(futurestestfunc)
         f.delay(10)
         result = f(4, 5)
         b = f.cancel()
@@ -1099,7 +1124,7 @@ class TestFutures(unittest.TestCase):
         with self.assertRaises(RuntimeError) as x:
             result.value
         self.assertTrue("cancelled" in str(x.exception))
-        f = Pyro4.Future(futurestestfunc)
+        f = Pyro4.futures.Future(futurestestfunc)
         result = f(4, 5)
         result.value
         b = f.cancel()
