@@ -4,6 +4,7 @@ Tests for the daemon.
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
+from __future__ import print_function
 import sys
 import time
 import socket
@@ -14,9 +15,10 @@ import Pyro4.naming
 import Pyro4.constants
 import Pyro4.socketutil
 import Pyro4.message
-from Pyro4.util import get_serializer_by_id
-from Pyro4.errors import DaemonError, PyroError
-from Pyro4 import current_context
+import Pyro4.util
+from Pyro4.errors import DaemonError, PyroError, SerializeError
+from Pyro4.configuration import config
+from Pyro4.core import current_context
 from testsupport import *
 
 
@@ -43,29 +45,29 @@ class DaemonTests(unittest.TestCase):
     # 'on-line' tests are all taking place in another test, to keep this one simple.
 
     def setUp(self):
-        Pyro4.config.POLLTIMEOUT = 0.1
+        config.POLLTIMEOUT = 0.1
 
     def sendHandshakeMessage(self, conn, correlation_id=None):
-        ser = get_serializer_by_id(Pyro4.message.SERIALIZER_MARSHAL)
+        ser = Pyro4.util.get_serializer_by_id(Pyro4.util.MarshalSerializer.serializer_id)
         data, _ = ser.serializeData({"handshake": "hello", "object": Pyro4.constants.DAEMON_NAME}, False)
         annotations = {"CORR": correlation_id.bytes} if correlation_id else None
-        msg = Pyro4.message.Message(Pyro4.message.MSG_CONNECT, data, Pyro4.message.SERIALIZER_MARSHAL, 0, 99, annotations=annotations)
+        msg = Pyro4.message.Message(Pyro4.message.MSG_CONNECT, data, Pyro4.util.MarshalSerializer.serializer_id, 0, 99, annotations=annotations)
         msg.send(conn)
 
     def testSerializerConfig(self):
-        self.assertIsInstance(Pyro4.config.SERIALIZERS_ACCEPTED, set)
-        self.assertIsInstance(Pyro4.config.SERIALIZER, basestring)
-        self.assertGreater(len(Pyro4.config.SERIALIZERS_ACCEPTED), 1)
+        self.assertIsInstance(config.SERIALIZERS_ACCEPTED, set)
+        self.assertIsInstance(config.SERIALIZER, basestring)
+        self.assertGreater(len(config.SERIALIZERS_ACCEPTED), 1)
 
     def testSerializerAccepted(self):
-        self.assertIn("marshal", Pyro4.config.SERIALIZERS_ACCEPTED)
-        self.assertNotIn("pickle", Pyro4.config.SERIALIZERS_ACCEPTED)
-        self.assertNotIn("dill", Pyro4.config.SERIALIZERS_ACCEPTED)
+        self.assertIn("marshal", config.SERIALIZERS_ACCEPTED)
+        self.assertNotIn("pickle", config.SERIALIZERS_ACCEPTED)
+        self.assertNotIn("dill", config.SERIALIZERS_ACCEPTED)
         with Pyro4.core.Daemon(port=0) as d:
-            msg = Pyro4.message.Message(Pyro4.message.MSG_INVOKE, b"", Pyro4.message.SERIALIZER_MARSHAL, 0, 0, hmac_key=d._pyroHmacKey)
+            msg = Pyro4.message.Message(Pyro4.message.MSG_INVOKE, b"", Pyro4.util.MarshalSerializer.serializer_id, 0, 0, hmac_key=d._pyroHmacKey)
             cm = ConnectionMock(msg)
             d.handleRequest(cm)  # marshal serializer should be accepted
-            msg = Pyro4.message.Message(Pyro4.message.MSG_INVOKE, b"", Pyro4.message.SERIALIZER_PICKLE, 0, 0, hmac_key=d._pyroHmacKey)
+            msg = Pyro4.message.Message(Pyro4.message.MSG_INVOKE, b"", Pyro4.util.PickleSerializer.serializer_id, 0, 0, hmac_key=d._pyroHmacKey)
             cm = ConnectionMock(msg)
             try:
                 d.handleRequest(cm)
@@ -73,17 +75,13 @@ class DaemonTests(unittest.TestCase):
             except Pyro4.errors.ProtocolError as x:
                 self.assertIn("serializer that is not accepted", str(x))
                 pass
-            import platform
-            if platform.python_implementation() in ('PyPy', 'IronPython'):
-                return
-            msg = Pyro4.message.Message(Pyro4.message.MSG_INVOKE, b"", Pyro4.message.SERIALIZER_DILL, 0, 0, hmac_key=d._pyroHmacKey)
+            msg = Pyro4.message.Message(Pyro4.message.MSG_INVOKE, b"", Pyro4.util.DillSerializer.serializer_id, 0, 0, hmac_key=d._pyroHmacKey)
             cm = ConnectionMock(msg)
             try:
                 d.handleRequest(cm)
                 self.fail("should crash")
             except Pyro4.errors.ProtocolError as x:
-                self.assertIn("serializer that is not accepted", str(x))
-                pass
+                self.assertTrue("no serializer available for id" in str(x) or "serializer that is not accepted" in str(x))
 
     def testDaemon(self):
         with Pyro4.core.Daemon(port=0) as d:
@@ -92,7 +90,7 @@ class DaemonTests(unittest.TestCase):
             self.assertIn(Pyro4.constants.DAEMON_NAME, d.objectsById)
             self.assertEqual("PYRO:" + Pyro4.constants.DAEMON_NAME + "@" + d.locationStr, str(d.uriFor(Pyro4.constants.DAEMON_NAME)))
             # check the string representations
-            expected = "<Pyro4.core.Daemon at 0x%x; %s; 1 objects>" % (id(d), d.locationStr)
+            expected = "<Pyro4.core.Daemon at 0x%x; %s - %s; 1 objects>" % (id(d), d.locationStr, Pyro4.socketutil.family_str(d.sock))
             self.assertEqual(expected, str(d))
             self.assertEqual(expected, unicode(d))
             self.assertEqual(expected, repr(d))
@@ -120,7 +118,7 @@ class DaemonTests(unittest.TestCase):
             self.assertEqual(locationstr, d.locationStr)
             self.assertEqual("PYRO:" + Pyro4.constants.DAEMON_NAME + "@" + locationstr, str(d.uriFor(Pyro4.constants.DAEMON_NAME)))
             # check the string representations
-            expected = "<Pyro4.core.Daemon at 0x%x; %s; 1 objects>" % (id(d), locationstr)
+            expected = "<Pyro4.core.Daemon at 0x%x; %s - Unix; 1 objects>" % (id(d), locationstr)
             self.assertEqual(expected, str(d))
             self.assertEqual(SOCKNAME, d.sock.getsockname())
             self.assertEqual(socket.AF_UNIX, d.sock.family)
@@ -133,33 +131,33 @@ class DaemonTests(unittest.TestCase):
             self.assertEqual(locationstr, d.locationStr)
             self.assertEqual("PYRO:" + Pyro4.constants.DAEMON_NAME + "@" + locationstr, str(d.uriFor(Pyro4.constants.DAEMON_NAME)))
             # check the string representations
-            expected = "<Pyro4.core.Daemon at 0x%x; %s; 1 objects>" % (id(d), locationstr)
+            expected = "<Pyro4.core.Daemon at 0x%x; %s - Unix; 1 objects>" % (id(d), locationstr)
             self.assertEqual(expected, str(d))
             sn_bytes = tobytes(SOCKNAME)
             self.assertEqual(sn_bytes, d.sock.getsockname())
             self.assertEqual(socket.AF_UNIX, d.sock.family)
 
     def testServertypeThread(self):
-        old_servertype = Pyro4.config.SERVERTYPE
-        Pyro4.config.SERVERTYPE = "thread"
+        old_servertype = config.SERVERTYPE
+        config.SERVERTYPE = "thread"
         with Pyro4.core.Daemon(port=0) as d:
             self.assertIn(d.sock, d.sockets, "daemon's socketlist should contain the server socket")
             self.assertTrue(len(d.sockets) == 1, "daemon without connections should have just 1 socket")
-        Pyro4.config.SERVERTYPE = old_servertype
+        config.SERVERTYPE = old_servertype
 
     def testServertypeMultiplex(self):
-        old_servertype = Pyro4.config.SERVERTYPE
-        Pyro4.config.SERVERTYPE = "multiplex"
+        old_servertype = config.SERVERTYPE
+        config.SERVERTYPE = "multiplex"
         with Pyro4.core.Daemon(port=0) as d:
             self.assertIn(d.sock, d.sockets, "daemon's socketlist should contain the server socket")
             self.assertTrue(len(d.sockets) == 1, "daemon without connections should have just 1 socket")
-        Pyro4.config.SERVERTYPE = old_servertype
+        config.SERVERTYPE = old_servertype
 
     def testServertypeFoobar(self):
-        old_servertype = Pyro4.config.SERVERTYPE
-        Pyro4.config.SERVERTYPE = "foobar"
+        old_servertype = config.SERVERTYPE
+        config.SERVERTYPE = "foobar"
         self.assertRaises(PyroError, Pyro4.core.Daemon)
-        Pyro4.config.SERVERTYPE = old_servertype
+        config.SERVERTYPE = old_servertype
 
     def testRegisterTwice(self):
         with Pyro4.core.Daemon(port=0) as d:
@@ -399,7 +397,7 @@ class DaemonTests(unittest.TestCase):
             self.assertEqual(Pyro4.message.MSG_CONNECTFAIL, msg.type)
             self.assertEqual(99, msg.seq)
             self.assertTrue(b"handshake fail validation error" in msg.data)
-        with Pyro4.Daemon(port=0) as d:
+        with Pyro4.core.Daemon(port=0) as d:
             self.sendHandshakeMessage(conn)
             success = d._handshake(conn, denied_reason="no way, handshake denied")
             self.assertFalse(success)
@@ -414,9 +412,7 @@ class DaemonTests(unittest.TestCase):
             def validateHandshake(self, conn, data):
                 return ["sure", "have", "fun"]
             def annotations(self):
-                ann = super(CustomHandshakeDaemon, self).annotations()
-                ann["XYZZ"] = b"custom annotation set by daemon"
-                return ann
+                return {"XYZZ": b"custom annotation set by daemon"}
         with CustomHandshakeDaemon(port=0) as d:
             corr_id = uuid.uuid4()
             self.sendHandshakeMessage(conn, correlation_id=corr_id)
@@ -430,7 +426,7 @@ class DaemonTests(unittest.TestCase):
             self.assertEqual(2, len(msg.annotations))
             self.assertEqual(corr_id.bytes, msg.annotations["CORR"])
             self.assertEqual(b"custom annotation set by daemon", msg.annotations["XYZZ"])
-            ser = get_serializer_by_id(msg.serializer_id)
+            ser = Pyro4.util.get_serializer_by_id(msg.serializer_id)
             data = ser.deserializeData(msg.data, msg.flags & Pyro4.message.FLAGS_COMPRESSED)
             self.assertEqual(["sure", "have", "fun"], data)
 
@@ -463,9 +459,9 @@ class DaemonTests(unittest.TestCase):
             pass
 
     def testNATzeroPort(self):
-        servertype = Pyro4.config.SERVERTYPE
+        servertype = config.SERVERTYPE
         try:
-            Pyro4.config.SERVERTYPE = "multiplex"
+            config.SERVERTYPE = "multiplex"
             with Pyro4.core.Daemon(nathost="nathosttest", natport=99999) as d:
                 host, port = d.locationStr.split(":")
                 self.assertNotEqual(99999, port)
@@ -473,7 +469,7 @@ class DaemonTests(unittest.TestCase):
             with Pyro4.core.Daemon(nathost="nathosttest", natport=0) as d:
                 host, port = d.locationStr.split(":")
                 self.assertEqual("nathosttest:%s" % port, d.natLocationStr)
-            Pyro4.config.SERVERTYPE = "thread"
+            config.SERVERTYPE = "thread"
             with Pyro4.core.Daemon(nathost="nathosttest", natport=99999) as d:
                 host, port = d.locationStr.split(":")
                 self.assertNotEqual(99999, port)
@@ -482,21 +478,21 @@ class DaemonTests(unittest.TestCase):
                 host, port = d.locationStr.split(":")
                 self.assertEqual("nathosttest:%s" % port, d.natLocationStr)
         finally:
-            Pyro4.config.SERVERTYPE = servertype
+            config.SERVERTYPE = servertype
 
     def testNATconfig(self):
         try:
-            Pyro4.config.NATHOST = None
-            Pyro4.config.NATPORT = 0
+            config.NATHOST = None
+            config.NATPORT = 0
             with Pyro4.core.Daemon() as d:
                 self.assertIsNone(d.natLocationStr)
-            Pyro4.config.NATHOST = "nathosttest"
-            Pyro4.config.NATPORT = 12345
+            config.NATHOST = "nathosttest"
+            config.NATPORT = 12345
             with Pyro4.core.Daemon() as d:
                 self.assertEqual("nathosttest:12345", d.natLocationStr)
         finally:
-            Pyro4.config.NATHOST = None
-            Pyro4.config.NATPORT = 0
+            config.NATHOST = None
+            config.NATPORT = 0
 
     def testBehaviorDefaults(self):
         class TestClass:
@@ -606,7 +602,7 @@ class DaemonTests(unittest.TestCase):
         d1.close()
         d2.close()
         try:
-            Pyro4.config.SERVERTYPE = "multiplex"
+            config.SERVERTYPE = "multiplex"
             d1 = Pyro4.core.Daemon()
             d2 = Pyro4.core.Daemon()
             nsuri, nsd, bcd = Pyro4.naming.startNS(host="", bchost="")
@@ -628,7 +624,7 @@ class DaemonTests(unittest.TestCase):
             d2.close()
             d1.close()
         finally:
-            Pyro4.config.SERVERTYPE = "thread"
+            config.SERVERTYPE = "thread"
 
 
 class MetaInfoTests(unittest.TestCase):
@@ -644,18 +640,26 @@ class MetaInfoTests(unittest.TestCase):
         with Pyro4.core.Daemon() as d:
             daemon_obj = d.objectsById[Pyro4.constants.DAEMON_NAME]
             meta = daemon_obj.get_metadata(Pyro4.constants.DAEMON_NAME)
-            for ser_id in [Pyro4.message.SERIALIZER_JSON, Pyro4.message.SERIALIZER_MARSHAL, Pyro4.message.SERIALIZER_PICKLE, Pyro4.message.SERIALIZER_SERPENT, Pyro4.message.SERIALIZER_DILL]:
-                import platform
-                if platform.python_implementation() in ('IronPython', 'PyPy') and \
-                   ser_id == Pyro4.message.SERIALIZER_DILL:
-                    continue
-                serializer = get_serializer_by_id(ser_id)
+            for ser_id in [Pyro4.util.JsonSerializer.serializer_id,
+                           Pyro4.util.MarshalSerializer.serializer_id,
+                           Pyro4.util.PickleSerializer.serializer_id,
+                           Pyro4.util.SerpentSerializer.serializer_id]:
+                serializer = Pyro4.util.get_serializer_by_id(ser_id)
+                data = serializer.dumps(meta)
+                _ = serializer.loads(data)
+            try:
+                serializer = Pyro4.util.get_serializer_by_id(Pyro4.util.DillSerializer.serializer_id)
+            except SerializeError:
+                # dill doesn't work with ironpython so we allow an error here in that case
+                if sys.platform != "cli":
+                    raise
+            else:
                 data = serializer.dumps(meta)
                 _ = serializer.loads(data)
 
     def testMetaResetCache(self):
         class Dummy:
-            @Pyro4.expose
+            @Pyro4.core.expose
             def method(self):
                 pass
         with Pyro4.core.Daemon() as d:
@@ -665,13 +669,13 @@ class MetaInfoTests(unittest.TestCase):
             meta = daemon_obj.get_metadata(uri.object)
             self.assertNotIn("newly_added_method", meta["methods"])
             self.assertNotIn("newly_added_method_two", meta["methods"])
-            Dummy.newly_added_method = Pyro4.expose(lambda self: None)
+            Dummy.newly_added_method = Pyro4.core.expose(lambda self: None)
             meta = daemon_obj.get_metadata(uri.object)
             self.assertNotIn("newly_added_method", meta["methods"])
             d.resetMetadataCache(uri.object)
             meta = daemon_obj.get_metadata(uri.object)
             self.assertIn("newly_added_method", meta["methods"])
-            Dummy.newly_added_method_two = Pyro4.expose(lambda self: None)
+            Dummy.newly_added_method_two = Pyro4.core.expose(lambda self: None)
             d.resetMetadataCache(dummy)
             meta = daemon_obj.get_metadata(uri.object)
             self.assertIn("newly_added_method_two", meta["methods"])
