@@ -12,11 +12,13 @@ import pickle
 import base64
 import unittest
 import serpent
+import math
+import uuid
 import Pyro4.util
 import Pyro4.errors
 import Pyro4.core
 import Pyro4.futures
-import Pyro4.message
+from Pyro4.configuration import config
 from testsupport import *
 
 
@@ -24,16 +26,16 @@ class SerializeTests_pickle(unittest.TestCase):
     SERIALIZER = "pickle"
 
     def setUp(self):
-        self.previous_serializer = Pyro4.config.SERIALIZER
-        Pyro4.config.SERIALIZER = self.SERIALIZER
-        self.ser = Pyro4.util.get_serializer(Pyro4.config.SERIALIZER)
-        Pyro4.config.REQUIRE_EXPOSE = True
+        self.previous_serializer = config.SERIALIZER
+        config.SERIALIZER = self.SERIALIZER
+        self.ser = Pyro4.util.get_serializer(config.SERIALIZER)
+        config.REQUIRE_EXPOSE = True
 
     def tearDown(self):
-        Pyro4.config.SERIALIZER = self.previous_serializer
+        config.SERIALIZER = self.previous_serializer
 
     def testSerItself(self):
-        s = Pyro4.util.get_serializer(Pyro4.config.SERIALIZER)
+        s = Pyro4.util.get_serializer(config.SERIALIZER)
         p, _ = self.ser.serializeData(s)
         s2 = self.ser.deserializeData(p)
         self.assertEqual(s, s2)
@@ -42,11 +44,8 @@ class SerializeTests_pickle(unittest.TestCase):
 
     def testSerUnicode(self):
         data = unicode("x")
-        ser, _ = self.ser.serializeData(data)
-        expected_type = str if sys.platform == "cli" else bytes  # ironpython serializes into str, not bytes... :(
-        self.assertTrue(type(ser) is expected_type)
-        ser, _ = self.ser.serializeCall(data, unicode("method"), [], {})
-        self.assertTrue(type(ser) is expected_type)
+        self.ser.serializeData(data)
+        self.ser.serializeCall(data, unicode("method"), [], {})
 
     def testSerCompression(self):
         d1, c1 = self.ser.serializeData("small data", compress=True)
@@ -150,8 +149,10 @@ class SerializeTests_pickle(unittest.TestCase):
             d, _ = self.ser.serializeData(daemon)
             d2 = self.ser.deserializeData(d)
             self.assertTrue(len(d2.__dict__) == 0, "deserialized daemon should be empty")
+            self.assertTrue("Pyro4.core.Daemon" in repr(d2))
+            self.assertTrue("unusable" in repr(d2))
             try:
-                Pyro4.config.AUTOPROXY = False
+                config.AUTOPROXY = False
                 obj = pprint.PrettyPrinter(stream="dummy", width=42)
                 obj.name = "hello"
                 daemon.register(obj)
@@ -162,7 +163,7 @@ class SerializeTests_pickle(unittest.TestCase):
                     self.assertEqual("hello", o2.name)
                     self.assertEqual(42, o2._width)
             finally:
-                Pyro4.config.AUTOPROXY = True
+                config.AUTOPROXY = True
 
     def testPyroClasses(self):
         uri = Pyro4.core.URI("PYRO:object@host:4444")
@@ -170,6 +171,8 @@ class SerializeTests_pickle(unittest.TestCase):
         x = self.ser.deserializeData(s, c)
         self.assertIsInstance(x, Pyro4.core.URI)
         self.assertEqual(uri, x)
+        self.assertTrue("Pyro4.core.URI" in repr(uri))
+        self.assertEqual("PYRO:object@host:4444", str(uri))
         uri = Pyro4.core.URI("PYRO:12345@./u:/tmp/socketname")
         s, c = self.ser.serializeData(uri)
         x = self.ser.deserializeData(s, c)
@@ -196,15 +199,23 @@ class SerializeTests_pickle(unittest.TestCase):
         self.assertEqual("serializer", x._pyroSerializer)
         self.assertEqual(0, x._pyroTimeout, "must be reset to defaults")
         self.assertEqual(0, x._pyroMaxRetries, "must be reset to defaults")
+        self.assertTrue("Pyro4.core.Proxy" in repr(x))
+        self.assertTrue("Pyro4.core.Proxy" in str(x))
         daemon = Pyro4.core.Daemon()
         s, c = self.ser.serializeData(daemon)
         x = self.ser.deserializeData(s, c)
         self.assertIsInstance(x, Pyro4.core.Daemon)
+        self.assertTrue("Pyro4.core.Daemon" in repr(x))
+        self.assertTrue("unusable" in repr(x))
+        self.assertTrue("Pyro4.core.Daemon" in str(x))
+        self.assertTrue("unusable" in str(x))
         wrapper = Pyro4.futures._ExceptionWrapper(ZeroDivisionError("divided by zero"))
         s, c = self.ser.serializeData(wrapper)
         x = self.ser.deserializeData(s, c)
         self.assertIsInstance(x, Pyro4.futures._ExceptionWrapper)
         self.assertEqual("divided by zero", str(x.exception))
+        self.assertTrue("ExceptionWrapper" in repr(x))
+        self.assertTrue("ExceptionWrapper" in str(x))
 
     def testPyroClassesForDict(self):
         uri = Pyro4.core.URI("PYRO:object@host:4444")
@@ -346,12 +357,23 @@ class SerializeTests_pickle(unittest.TestCase):
     def testData(self):
         data = [42, "hello"]
         ser, compressed = self.ser.serializeData(data)
-        expected_type = str if sys.platform == "cli" else bytes  # ironpython serializes into str, not bytes... :(
-        self.assertTrue(type(ser) is expected_type)
         self.assertFalse(compressed)
         data2 = self.ser.deserializeData(ser, compressed=False)
         self.assertEqual(data, data2)
 
+    def testUnicodeData(self):
+        data = u"euro\u20aclowbytes\u0000\u0001\u007f\u0080\u00ff"
+        ser, compressed = self.ser.serializeData(data)
+        data2 = self.ser.deserializeData(ser, compressed=compressed)
+        self.assertEqual(data, data2)
+
+    def testUUID(self):
+        data = uuid.uuid1()
+        ser, compressed = self.ser.serializeData(data)
+        data2 = self.ser.deserializeData(ser, compressed=compressed)
+        uuid_as_str = str(data)
+        self.assertTrue(data2==data or data2==uuid_as_str)
+
     def testSet(self):
         data = {111, 222, 333}
         ser, compressed = self.ser.serializeData(data)
@@ -359,7 +381,7 @@ class SerializeTests_pickle(unittest.TestCase):
         self.assertEqual(data, data2)
 
     def testCircular(self):
-        data = [42, "hello", Pyro4.Proxy("PYRO:dummy@dummy:4444")]
+        data = [42, "hello", Pyro4.core.Proxy("PYRO:dummy@dummy:4444")]
         data.append(data)
         ser, compressed = self.ser.serializeData(data)
         data2 = self.ser.deserializeData(ser, compressed)
@@ -452,12 +474,12 @@ class SerializeTests_pickle(unittest.TestCase):
         self.assertEqual(Pyro4.core.URI("PYRO:555@localhost:80"), uri["uri"])
 
     def testProtocolVersion(self):
-        self.assertGreaterEqual(Pyro4.config.PICKLE_PROTOCOL_VERSION, 2)
-        self.assertEqual(pickle.HIGHEST_PROTOCOL, Pyro4.config.PICKLE_PROTOCOL_VERSION)
+        self.assertGreaterEqual(config.PICKLE_PROTOCOL_VERSION, 2)
+        self.assertEqual(pickle.HIGHEST_PROTOCOL, config.PICKLE_PROTOCOL_VERSION)
 
     def testUriSerializationWithoutSlots(self):
-        orig_protocol = Pyro4.config.PICKLE_PROTOCOL_VERSION
-        Pyro4.config.PICKLE_PROTOCOL_VERSION = 2
+        orig_protocol = config.PICKLE_PROTOCOL_VERSION
+        config.PICKLE_PROTOCOL_VERSION = 2
         try:
             u = Pyro4.core.URI("PYRO:obj@localhost:1234")
             d, compr = self.ser.serializeData(u)
@@ -468,7 +490,7 @@ class SerializeTests_pickle(unittest.TestCase):
             result2 = b'\x80\x02cPyro4.core\nURI\n)\x81(X\x04\x00\x00\x00PYROX\x03\x00\x00\x00objNX\t\x00\x00\x00localhostM\xd2\x04tb.'
             self.assertTrue(d in (result1, result2))
         finally:
-            Pyro4.config.PICKLE_PROTOCOL_VERSION = orig_protocol
+            config.PICKLE_PROTOCOL_VERSION = orig_protocol
 
     def testFloatPrecision(self):
         f1 = 1482514078.54635912345
@@ -479,16 +501,87 @@ class SerializeTests_pickle(unittest.TestCase):
         v = self.ser.deserializeData(d, compr)
         self.assertEqual(floats, v, "float precision must not be compromised in any serializer")
 
+    def testSourceByteTypes_deserialize(self):
+        # uncompressed
+        call_ser, _ = self.ser.serializeCall("object", "method", [1, 2, 3], {"kwarg": 42}, False)
+        ser, _ = self.ser.serializeData([4, 5, 6], False)
+        _, _, vargs, _ = self.ser.deserializeCall(bytearray(call_ser), False)
+        self.assertEqual([1, 2, 3], vargs)
+        d = self.ser.deserializeData(bytearray(ser), False)
+        self.assertEqual([4, 5, 6], d)
+        if sys.version_info < (3, 0):
+            _, _, vargs, _ = self.ser.deserializeCall(buffer(call_ser), False)
+            self.assertEqual([1, 2, 3], vargs)
+            d = self.ser.deserializeData(buffer(ser), False)
+            self.assertEqual([4, 5, 6], d)
+        # compressed
+        call_ser, _ = self.ser.serializeCall("object", "method", [1, 2, 3]*100, {"kwarg": 42}, True)
+        ser, _ = self.ser.serializeData([4, 5, 6]*100, True)
+        _, _, vargs, _ = self.ser.deserializeCall(bytearray(call_ser), True)
+        self.assertEqual(300, len(vargs))
+        d = self.ser.deserializeData(bytearray(ser), True)
+        self.assertEqual(300, len(d))
+        if sys.version_info < (3, 0):
+            _, _, vargs, _ = self.ser.deserializeCall(buffer(call_ser), True)
+            self.assertEqual(300, len(vargs))
+            d = self.ser.deserializeData(buffer(ser), True)
+            self.assertEqual(300, len(d))
 
-import platform
-@unittest.skipIf(platform.python_implementation() in ('PyPy', 'IronPython'),
-                 'PyPy and IronPython not currently supported with dill')
+    @unittest.skipIf(sys.platform == "cli", "ironpython can't properly create memoryviews from serialized data")
+    def testSourceByteTypes_deserialize_memoryview(self):
+        # uncompressed
+        call_ser, _ = self.ser.serializeCall("object", "method", [1, 2, 3], {"kwarg": 42}, False)
+        ser, _ = self.ser.serializeData([4, 5, 6], False)
+        _, _, vargs, _ = self.ser.deserializeCall(memoryview(call_ser), False)
+        self.assertEqual([1, 2, 3], vargs)
+        d = self.ser.deserializeData(memoryview(ser), False)
+        self.assertEqual([4, 5, 6], d)
+        # compressed
+        call_ser, _ = self.ser.serializeCall("object", "method", [1, 2, 3]*100, {"kwarg": 42}, True)
+        ser, _ = self.ser.serializeData([4, 5, 6]*100, True)
+        _, _, vargs, _ = self.ser.deserializeCall(memoryview(call_ser), True)
+        self.assertEqual(300, len(vargs))
+        d = self.ser.deserializeData(memoryview(ser), True)
+        self.assertEqual(300, len(d))
+
+    def testSourceByteTypes_loads(self):
+        call_ser, _ = self.ser.serializeCall("object", "method", [1, 2, 3], {"kwarg": 42}, False)
+        ser, _ = self.ser.serializeData([4, 5, 6], False)
+        _, _, vargs, _ = self.ser.loadsCall(bytearray(call_ser))
+        self.assertEqual([1, 2, 3], vargs)
+        d = self.ser.loads(bytearray(ser))
+        self.assertEqual([4, 5, 6], d)
+        if sys.version_info < (3, 0):
+            _, _, vargs, _ = self.ser.loadsCall(buffer(call_ser))
+            self.assertEqual([1, 2, 3], vargs)
+            d = self.ser.loads(buffer(ser))
+            self.assertEqual([4, 5, 6], d)
+
+    @unittest.skipIf(sys.platform == "cli", "ironpython can't properly create memoryviews from serialized data")
+    def testSourceByteTypes_loads_memoryview(self):
+        call_ser, _ = self.ser.serializeCall("object", "method", [1, 2, 3], {"kwarg": 42}, False)
+        ser, _ = self.ser.serializeData([4, 5, 6], False)
+        _, _, vargs, _ = self.ser.loadsCall(memoryview(call_ser))
+        self.assertEqual([1, 2, 3], vargs)
+        d = self.ser.loads(memoryview(ser))
+        self.assertEqual([4, 5, 6], d)
+
+
+is_ironpython_without_dill = False
+try:
+    import dill
+except ImportError:
+    if sys.platform == "cli":
+        is_ironpython_without_dill = True
+
+
+@unittest.skipIf(is_ironpython_without_dill, "dill with ironpython has issues so it's fine if we don't test this")
 class SerializeTests_dill(SerializeTests_pickle):
     SERIALIZER = "dill"
 
     def testProtocolVersion(self):
         import dill
-        self.assertEqual(dill.HIGHEST_PROTOCOL, Pyro4.config.DILL_PROTOCOL_VERSION)
+        self.assertEqual(dill.HIGHEST_PROTOCOL, config.DILL_PROTOCOL_VERSION)
 
     @unittest.skip('not implemented')
     def testUriSerializationWithoutSlots(self):
@@ -511,10 +604,6 @@ class SerializeTests_dill(SerializeTests_pickle):
 class SerializeTests_serpent(SerializeTests_pickle):
     SERIALIZER = "serpent"
 
-    @unittest.skip("pickle specific")
-    def testProtocolVersion(self):
-        pass
-
     def testCircular(self):
         with self.assertRaises(ValueError):  # serpent doesn't support object graphs (since serpent 1.7 reports ValueError instead of crashing)
             super(SerializeTests_serpent, self).testCircular()
@@ -566,10 +655,6 @@ class SerializeTests_serpent(SerializeTests_pickle):
 class SerializeTests_json(SerializeTests_pickle):
     SERIALIZER = "json"
 
-    @unittest.skip("pickle specific")
-    def testProtocolVersion(self):
-        pass
-
     def testCircular(self):
         with self.assertRaises(ValueError):  # json doesn't support object graphs
             super(SerializeTests_json, self).testCircular()
@@ -602,6 +687,25 @@ class SerializeTests_marshal(SerializeTests_pickle):
         pass
 
 
+class SerializeTests_msgpack(SerializeTests_pickle):
+    SERIALIZER = "msgpack"
+
+    @unittest.skip("circular will crash msgpack")
+    def testCircular(self):
+        pass
+
+    def testSet(self):
+        # msgpack serializes a set into a list, so we override this
+        data = {111, 222, 333}
+        ser, compressed = self.ser.serializeData(data)
+        data2 = self.ser.deserializeData(ser, compressed=compressed)
+        self.assertEqual(list(data), data2)
+
+    @unittest.skip("msgpack is implementation dependent")
+    def testUriSerializationWithoutSlots(self):
+        pass
+
+
 class GenericTests(unittest.TestCase):
     def testSerializersAvailable(self):
         Pyro4.util.get_serializer("pickle")
@@ -617,21 +721,27 @@ class GenericTests(unittest.TestCase):
         except ImportError:
             pass
         try:
-            import platform
-            if platform.python_implementation() in ('PyPy', 'IronPython'):
-                raise ImportError('Currently dill is not supported with IronPython and PyPy')
             import dill
             Pyro4.util.get_serializer("dill")
         except ImportError:
             pass
 
+    def testAssignedSerializerIds(self):
+        self.assertEqual(1, Pyro4.util.SerpentSerializer.serializer_id)
+        self.assertEqual(2, Pyro4.util.JsonSerializer.serializer_id)
+        self.assertEqual(3, Pyro4.util.MarshalSerializer.serializer_id)
+        self.assertEqual(4, Pyro4.util.PickleSerializer.serializer_id)
+        self.assertEqual(5, Pyro4.util.DillSerializer.serializer_id)
+        self.assertEqual(6, Pyro4.util.MsgpackSerializer.serializer_id)
+
     def testSerializersAvailableById(self):
-        Pyro4.util.get_serializer_by_id(Pyro4.message.SERIALIZER_PICKLE)
-        import platform
-        if platform.python_implementation() not in ('PyPy', 'IronPython'):
-            Pyro4.util.get_serializer_by_id(Pyro4.message.SERIALIZER_DILL)
-        Pyro4.util.get_serializer_by_id(Pyro4.message.SERIALIZER_MARSHAL)
-        self.assertRaises(Pyro4.errors.ProtocolError, lambda: Pyro4.util.get_serializer_by_id(9999999))
+        Pyro4.util.get_serializer_by_id(1)  # serpent
+        Pyro4.util.get_serializer_by_id(2)  # json
+        Pyro4.util.get_serializer_by_id(3)  # marshal
+        Pyro4.util.get_serializer_by_id(4)  # pickle
+        # ids 5 and 6 (dill, msgpack) are not always available, so we skip those.
+        self.assertRaises(Pyro4.errors.SerializeError, lambda: Pyro4.util.get_serializer_by_id(0))
+        self.assertRaises(Pyro4.errors.SerializeError, lambda: Pyro4.util.get_serializer_by_id(7))
 
     def testDictClassFail(self):
         o = pprint.PrettyPrinter(stream="dummy", width=42)
@@ -731,6 +841,16 @@ class GenericTests(unittest.TestCase):
             _ = Pyro4.util.SerializerBase.dict_to_class(data)
         self.assertEqual("unsupported serialized class: builtins.ZeroDivisionError", str(cm.exception))
 
+    def testWeirdFloats(self):
+        ser = Pyro4.util.get_serializer(config.SERIALIZER)
+        p, _ = ser.serializeData([float("+inf"), float("-inf"), float("nan")])
+        s2 = ser.deserializeData(p)
+        self.assertTrue(math.isinf(s2[0]))
+        self.assertEqual(1.0, math.copysign(1, s2[0]))
+        self.assertTrue(math.isinf(s2[1]))
+        self.assertEqual(-1.0, math.copysign(1, s2[1]))
+        self.assertTrue(math.isnan(s2[2]))
+
 
 def mything_dict(obj):
     return {
