@@ -4,19 +4,23 @@ Tests for a running Pyro server, without timeouts.
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
+from __future__ import print_function
 import time
 import sys
+import threading
 import uuid
 import unittest
+import serpent
 import Pyro4.core
 import Pyro4.errors
 import Pyro4.util
 import Pyro4.message
-from Pyro4 import threadutil, current_context
+import Pyro4.socketutil
+from Pyro4.configuration import config
 from testsupport import *
 
 
-@Pyro4.expose
+@Pyro4.core.expose
 class ServerTestObject(object):
     something = 99
     dict_attr = {}
@@ -44,7 +48,10 @@ class ServerTestObject(object):
     def echo(self, obj):
         return obj
 
-    @Pyro4.oneway
+    def blob(self, blob):
+        return blob.info, blob.deserialized()
+
+    @Pyro4.core.oneway
     def oneway_delay(self, delay):
         time.sleep(delay)
 
@@ -62,7 +69,7 @@ class ServerTestObject(object):
     def nonserializableException(self):
         raise NonserializableError(("xantippe", lambda x: 0))
 
-    @Pyro4.oneway
+    @Pyro4.core.oneway
     def oneway_multiply(self, x, y):
         return x * y
 
@@ -85,13 +92,24 @@ class ServerTestObject(object):
         yield "one"
         yield "two"
         yield "three"
+        yield "four"
+        yield "five"
+
+    def response_annotation(self):
+        # part of the annotations tests
+        if "XYZZ" not in Pyro4.core.current_context.annotations:
+            raise ValueError("XYZZ should be present in annotations in the daemon")
+        if Pyro4.core.current_context.annotations["XYZZ"] != b"data from proxy via new api":
+            raise ValueError("XYZZ annotation has wrong data")
+        Pyro4.core.current_context.response_annotations["ANN2"] = b"daemon annotation via new api"
+        return {"annotations_in_daemon": Pyro4.core.current_context.annotations}
 
 
 class NotEverythingExposedClass(object):
     def __init__(self, name):
         self.name = name
 
-    @Pyro4.expose
+    @Pyro4.core.expose
     def getName(self):
         return self.name
 
@@ -99,12 +117,12 @@ class NotEverythingExposedClass(object):
         return "you should not see this"    # .... only when REQUIRE_EXPOSE is set to True is this valid
 
 
-class DaemonLoopThread(threadutil.Thread):
+class DaemonLoopThread(threading.Thread):
     def __init__(self, pyrodaemon):
         super(DaemonLoopThread, self).__init__()
         self.setDaemon(True)
         self.pyrodaemon = pyrodaemon
-        self.running = threadutil.Event()
+        self.running = threading.Event()
         self.running.clear()
 
     def run(self):
@@ -129,8 +147,8 @@ class DaemonWithSabotagedHandshake(Pyro4.core.Daemon):
 
 class ServerTestsBrokenHandshake(unittest.TestCase):
     def setUp(self):
-        Pyro4.config.LOGWIRE = True
-        Pyro4.config.SERIALIZERS_ACCEPTED.add("pickle")
+        config.LOGWIRE = True
+        config.SERIALIZERS_ACCEPTED.add("pickle")
         self.daemon = DaemonWithSabotagedHandshake(port=0)
         obj = ServerTestObject()
         uri = self.daemon.register(obj, "something")
@@ -144,18 +162,17 @@ class ServerTestsBrokenHandshake(unittest.TestCase):
         time.sleep(0.05)
         self.daemon.shutdown()
         self.daemonthread.join()
-        Pyro4.config.SERIALIZERS_ACCEPTED.discard("pickle")
+        config.SERIALIZERS_ACCEPTED.discard("pickle")
 
     def testDaemonConnectFail(self):
         # check what happens when the daemon responds with a failed connection msg
-        with Pyro4.Proxy(self.objectUri) as p:
+        with Pyro4.core.Proxy(self.objectUri) as p:
             try:
                 p.ping()
                 self.fail("expected CommunicationError")
-            except Pyro4.errors.CommunicationError:
-                xv = sys.exc_info()[1]
-                message = str(xv)
-                self.assertIn("reason:", message)
+            except Pyro4.errors.CommunicationError as x:
+                message = str(x)
+                self.assertIn("rejected:", message)
                 self.assertIn("rigged connection failure", message)
 
 
@@ -163,8 +180,8 @@ class ServerTestsOnce(unittest.TestCase):
     """tests that are fine to run with just a single server type"""
 
     def setUp(self):
-        Pyro4.config.LOGWIRE = True
-        Pyro4.config.SERIALIZERS_ACCEPTED.add("pickle")
+        config.LOGWIRE = True
+        config.SERIALIZERS_ACCEPTED.add("pickle")
         self.daemon = Pyro4.core.Daemon(port=0)
         obj = ServerTestObject()
         uri = self.daemon.register(obj, "something")
@@ -181,7 +198,7 @@ class ServerTestsOnce(unittest.TestCase):
         if self.daemon is not None:
             self.daemon.shutdown()
             self.daemonthread.join()
-        Pyro4.config.SERIALIZERS_ACCEPTED.discard("pickle")
+        config.SERIALIZERS_ACCEPTED.discard("pickle")
 
     def testPingMessage(self):
         with Pyro4.core.Proxy(self.objectUri) as p:
@@ -206,8 +223,8 @@ class ServerTestsOnce(unittest.TestCase):
 
     def testMetaOffAttrs(self):
         try:
-            old_meta = Pyro4.config.METADATA
-            Pyro4.config.METADATA = False
+            old_meta = config.METADATA
+            config.METADATA = False
             # should fail here, because there is no meta info about attributes
             with Pyro4.core.Proxy(self.objectUri) as p:
                 self.assertEqual(55, p.multiply(5, 11))
@@ -222,12 +239,12 @@ class ServerTestsOnce(unittest.TestCase):
                 x = p.getDict()
                 self.assertEqual({"number": 42}, x)
         finally:
-            Pyro4.config.METADATA = old_meta
+            config.METADATA = old_meta
 
     def testMetaOnAttrs(self):
         try:
-            old_meta = Pyro4.config.METADATA
-            Pyro4.config.METADATA = True
+            old_meta = config.METADATA
+            config.METADATA = True
             with Pyro4.core.Proxy(self.objectUri) as p:
                 self.assertEqual(55, p.multiply(5, 11))
                 # property
@@ -241,7 +258,7 @@ class ServerTestsOnce(unittest.TestCase):
                     # attribute should fail (meta only works for exposed properties)
                     p.dict_attr.update({"more": 666})
         finally:
-            Pyro4.config.METADATA = old_meta
+            config.METADATA = old_meta
 
     def testSomeArgumentTypes(self):
         with Pyro4.core.Proxy(self.objectUri) as p:
@@ -285,7 +302,7 @@ class ServerTestsOnce(unittest.TestCase):
             p._pyroBind()
             self.assertEqual({'value', 'dictionary'}, p._pyroAttrs)
             self.assertEqual({'echo', 'getDict', 'divide', 'nonserializableException', 'ping', 'oneway_delay', 'delayAndId', 'delay', 'testargs',
-                                  'multiply', 'oneway_multiply', 'getDictAttr', 'iterator', 'generator'}, p._pyroMethods)
+                                  'multiply', 'oneway_multiply', 'getDictAttr', 'iterator', 'generator', 'response_annotation', 'blob'}, p._pyroMethods)
             self.assertEqual({'oneway_multiply', 'oneway_delay'}, p._pyroOneway)
             p._pyroAttrs = None
             p._pyroGetMetadata()
@@ -297,9 +314,16 @@ class ServerTestsOnce(unittest.TestCase):
             p._pyroGetMetadata(known_metadata={"attrs": set(), "oneway": set(), "methods": {"ping"}})
             self.assertEqual(set(), p._pyroAttrs)
 
+    def testProxyRepr(self):
+        with Pyro4.core.Proxy(self.objectUri) as p:
+            p._pyroBind()
+            expected = "<Pyro4.core.Proxy at 0x%x; connected %s; for %s>" % (id(p), Pyro4.socketutil.family_str(self.daemon.sock), self.objectUri)
+            self.assertEqual(expected, str(p))
+            self.assertEqual(expected, repr(p))
+
     def testProxyAttrsMetadataOff(self):
         try:
-            Pyro4.config.METADATA = False
+            config.METADATA = False
             # read attributes
             with Pyro4.core.Proxy(self.objectUri) as p:
                 a = p.multiply
@@ -313,11 +337,11 @@ class ServerTestsOnce(unittest.TestCase):
                 p.some_weird_attribute = 42
                 self.assertEqual(42, p.some_weird_attribute)
         finally:
-            Pyro4.config.METADATA = True
+            config.METADATA = True
 
     def testProxyAttrsMetadataOn(self):
         try:
-            Pyro4.config.METADATA = True
+            config.METADATA = True
             # read attributes
             with Pyro4.core.Proxy(self.objectUri) as p:
                 # unconnected proxy still has empty metadata.
@@ -334,44 +358,66 @@ class ServerTestsOnce(unittest.TestCase):
                 self.assertEqual(42, p.value)
                 self.assertTrue("value" in p._pyroAttrs)
         finally:
-            Pyro4.config.METADATA = True
+            config.METADATA = True
 
-    def testProxyAnnotations(self):
+    def testProxyAnnotationsOldApi(self):
         class CustomAnnotationsProxy(Pyro4.core.Proxy):
             def __init__(self, uri, response):
                 self.__dict__["response"] = response
                 super(CustomAnnotationsProxy, self).__init__(uri)
             def _pyroAnnotations(self):
-                ann = super(CustomAnnotationsProxy, self)._pyroAnnotations()
-                ann["XYZZ"] = b"some data"
+                ann = {"XYZZ": b"some data sent by old annotations api"}
                 self.__dict__["response"]["annotations_sent"] = ann
                 return ann
             def _pyroResponseAnnotations(self, annotations, msgtype):
                 self.__dict__["response"]["annotations"] = annotations
                 self.__dict__["response"]["msgtype"] = msgtype
         response = {}
-        corr_id = current_context.correlation_id = uuid.uuid4()
+        corr_id = Pyro4.core.current_context.correlation_id = uuid.uuid4()
         with CustomAnnotationsProxy(self.objectUri, response) as p:
             p.ping()
-        self.assertDictEqual({"CORR": corr_id.bytes, "XYZZ": b"some data"}, p.__dict__["response"]["annotations_sent"])
+        self.assertDictEqual({"XYZZ": b"some data sent by old annotations api"}, p.__dict__["response"]["annotations_sent"])
         self.assertEqual(Pyro4.message.MSG_RESULT, p.__dict__["response"]["msgtype"])
         self.assertDictEqual({"CORR": corr_id.bytes}, p.__dict__["response"]["annotations"])
 
+    def testProxyAnnotations(self):
+        class AnnotationsProxy(Pyro4.core.Proxy):
+            def _pyroAnnotations(self):
+                return {"QWER": b"data via old api"}
+        with AnnotationsProxy(self.objectUri) as p:
+            Pyro4.core.current_context.annotations = {"XYZZ": b"invalid test data"}
+            try:
+                p.response_annotation()
+                self.fail("should fail")
+            except ValueError:
+                pass
+            Pyro4.core.current_context.annotations = {"XYZZ": b"data from proxy via new api"}
+            response = p.response_annotation()
+            self.assertEqual(b"daemon annotation via new api", Pyro4.core.current_context.response_annotations["ANN2"])
+            # check that the daemon received both the old and the new annotation api data:
+            daemon_annotations = response["annotations_in_daemon"]
+            if sys.version_info < (3, 0) and sys.platform != "cli":
+                self.assertEqual("data from proxy via new api", daemon_annotations["XYZZ"])
+                self.assertEqual("data via old api", daemon_annotations["QWER"])
+            else:
+                self.assertEqual(b"data from proxy via new api", serpent.tobytes(daemon_annotations["XYZZ"]))
+                self.assertEqual(b"data via old api", serpent.tobytes(daemon_annotations["QWER"]))
+
     def testExposedNotRequired(self):
         try:
-            old_require = Pyro4.config.REQUIRE_EXPOSE
-            Pyro4.config.REQUIRE_EXPOSE = False
+            old_require = config.REQUIRE_EXPOSE
+            config.REQUIRE_EXPOSE = False
             with self.daemon.proxyFor("unexposed") as p:
                 self.assertEqual({"unexposed", "getName"}, p._pyroMethods)
                 self.assertEqual("hello", p.getName())
                 self.assertEqual("you should not see this", p.unexposed())   # you *should* see it when REQUIRE_EXPOSE is False :)
         finally:
-            Pyro4.config.REQUIRE_EXPOSE = old_require
+            config.REQUIRE_EXPOSE = old_require
 
     def testExposedRequired(self):
         try:
-            old_require = Pyro4.config.REQUIRE_EXPOSE
-            Pyro4.config.REQUIRE_EXPOSE = True
+            old_require = config.REQUIRE_EXPOSE
+            config.REQUIRE_EXPOSE = True
             with self.daemon.proxyFor("unexposed") as p:
                 self.assertEqual({"getName"}, p._pyroMethods)
                 self.assertEqual("hello", p.getName())
@@ -384,7 +430,7 @@ class ServerTestsOnce(unittest.TestCase):
                 expected_msg = "remote object '%s' has no exposed attribute 'unexposed_set'" % p._pyroUri
                 self.assertEqual(expected_msg, str(e.exception))
         finally:
-            Pyro4.config.REQUIRE_EXPOSE = old_require
+            config.REQUIRE_EXPOSE = old_require
 
     def testProperties(self):
         with Pyro4.core.Proxy(self.objectUri) as p:
@@ -402,7 +448,7 @@ class ServerTestsOnce(unittest.TestCase):
 
     def testHasAttr(self):
         try:
-            Pyro4.config.METADATA = False
+            config.METADATA = False
             with Pyro4.core.Proxy(self.objectUri) as p:
                 # with metadata off, all attributes are considered valid (and return a RemoteMethod object)
                 self.assertTrue(hasattr(p, "multiply"))
@@ -411,7 +457,7 @@ class ServerTestsOnce(unittest.TestCase):
                 self.assertTrue(hasattr(p, "_value"))
                 self.assertTrue(hasattr(p, "_dictionary"))
                 self.assertTrue(hasattr(p, "non_existing_attribute"))
-            Pyro4.config.METADATA = True
+            config.METADATA = True
             with Pyro4.core.Proxy(self.objectUri) as p:
                 # with metadata on, hasattr actually gives proper results
                 self.assertTrue(hasattr(p, "multiply"))
@@ -421,7 +467,7 @@ class ServerTestsOnce(unittest.TestCase):
                 self.assertFalse(hasattr(p, "_dictionary"))
                 self.assertFalse(hasattr(p, "non_existing_attribute"))
         finally:
-            Pyro4.config.METADATA = True
+            config.METADATA = True
 
     def testProxyMetadataKnown(self):
         with Pyro4.core.Proxy(self.objectUri) as p:
@@ -442,21 +488,19 @@ class ServerTestsOnce(unittest.TestCase):
             try:
                 p.nonserializableException()
                 self.fail("should crash")
-            except Exception:
-                xt, xv, tb = sys.exc_info()
-                self.assertTrue(issubclass(xt, Pyro4.errors.PyroError))
+            except Exception as x:
+                self.assertIsInstance(x, Pyro4.errors.PyroError)
                 tblines = "\n".join(Pyro4.util.getPyroTraceback())
                 self.assertTrue("unsupported serialized class" in tblines)
 
     def testNonserializableException_pickle(self):
         with Pyro4.core.Proxy(self.objectUri) as p:
-            Pyro4.config.SERIALIZER = "pickle"
+            config.SERIALIZER = "pickle"
             try:
                 p.nonserializableException()
                 self.fail("should crash")
-            except Exception:
-                xt, xv, tb = sys.exc_info()
-                self.assertTrue(issubclass(xt, Pyro4.errors.PyroError))
+            except Exception as x:
+                self.assertIsInstance(x, Pyro4.errors.PyroError)
                 tblines = "\n".join(Pyro4.util.getPyroTraceback())
                 self.assertTrue("PyroError: Error serializing exception" in tblines)
                 s1 = "Original exception: <class 'testsupport.NonserializableError'>:"
@@ -464,11 +508,11 @@ class ServerTestsOnce(unittest.TestCase):
                 self.assertTrue(s1 in tblines or s2 in tblines)
                 self.assertTrue("raise NonserializableError((\"xantippe" in tblines)
             finally:
-                Pyro4.config.SERIALIZER = "serpent"
+                config.SERIALIZER = "serpent"
 
     def testBatchProxy(self):
         with Pyro4.core.Proxy(self.objectUri) as p:
-            batch = Pyro4.batch(p)
+            batch = Pyro4.core.batch(p)
             self.assertIsNone(batch.multiply(7, 6))
             self.assertIsNone(batch.divide(999, 3))
             self.assertIsNone(batch.ping())
@@ -483,10 +527,10 @@ class ServerTestsOnce(unittest.TestCase):
 
     def testAsyncProxy(self):
         with Pyro4.core.Proxy(self.objectUri) as p:
-            async = Pyro4.async(p)
-            async._pyroBind()  # force that any metadata is processed
+            Pyro4.core.async(p)
+            p._pyroBind()  # force that any metadata is processed
             begin = time.time()
-            result = async.delayAndId(1, 42)
+            result = p.delayAndId(1, 42)
             duration = time.time() - begin
             self.assertTrue(duration < 0.1)
             self.assertFalse(result.ready)
@@ -497,18 +541,18 @@ class ServerTestsOnce(unittest.TestCase):
 
     def testAsyncProxyCallchain(self):
         class FuncHolder(object):
-            count = threadutil.AtomicCounter()
+            count = AtomicCounter()
 
             def function(self, value, increase=1):
                 self.count.incr()
                 return value + increase
 
         with Pyro4.core.Proxy(self.objectUri) as p:
-            async = Pyro4.async(p)
-            async._pyroBind()  # force that any metadata is processed
+            Pyro4.core.async(p)
+            p._pyroBind()  # force that any metadata is processed
             holder = FuncHolder()
             begin = time.time()
-            result = async.multiply(2, 3)
+            result = p.multiply(2, 3)
             result.then(holder.function, increase=10) \
                 .then(holder.function, increase=5) \
                 .then(holder.function)
@@ -521,7 +565,7 @@ class ServerTestsOnce(unittest.TestCase):
 
     def testBatchOneway(self):
         with Pyro4.core.Proxy(self.objectUri) as p:
-            batch = Pyro4.batch(p)
+            batch = Pyro4.core.batch(p)
             self.assertIsNone(batch.multiply(7, 6))
             self.assertIsNone(batch.delay(1))  # a delay shouldn't matter with oneway
             self.assertIsNone(batch.multiply(3, 4))
@@ -533,7 +577,7 @@ class ServerTestsOnce(unittest.TestCase):
 
     def testBatchAsync(self):
         with Pyro4.core.Proxy(self.objectUri) as p:
-            batch = Pyro4.batch(p)
+            batch = Pyro4.core.batch(p)
             self.assertIsNone(batch.multiply(7, 6))
             self.assertIsNone(batch.delay(1))  # a delay shouldn't matter with async
             self.assertIsNone(batch.multiply(3, 4))
@@ -549,7 +593,7 @@ class ServerTestsOnce(unittest.TestCase):
 
     def testBatchAsyncCallchain(self):
         class FuncHolder(object):
-            count = threadutil.AtomicCounter()
+            count = AtomicCounter()
 
             def function(self, values):
                 result = [value + 1 for value in values]
@@ -557,7 +601,7 @@ class ServerTestsOnce(unittest.TestCase):
                 return result
 
         with Pyro4.core.Proxy(self.objectUri) as p:
-            batch = Pyro4.batch(p)
+            batch = Pyro4.core.batch(p)
             self.assertIsNone(batch.multiply(7, 6))
             self.assertIsNone(batch.multiply(3, 4))
             result = batch(async=True)
@@ -582,7 +626,7 @@ class ServerTestsOnce(unittest.TestCase):
 
     def testPyroTracebackBatch(self):
         with Pyro4.core.Proxy(self.objectUri) as p:
-            batch = Pyro4.batch(p)
+            batch = Pyro4.core.batch(p)
             self.assertIsNone(batch.divide(999, 0))  # force an exception here
             results = batch()
             try:
@@ -598,10 +642,10 @@ class ServerTestsOnce(unittest.TestCase):
 
     def testAutoProxy(self):
         obj = ServerTestObject()
-        Pyro4.config.SERIALIZER = "pickle"
+        config.SERIALIZER = "pickle"
         try:
             with Pyro4.core.Proxy(self.objectUri) as p:
-                Pyro4.config.AUTOPROXY = False  # make sure autoproxy is disabled
+                config.AUTOPROXY = False  # make sure autoproxy is disabled
                 result = p.echo(obj)
                 self.assertIsInstance(result, ServerTestObject)
                 self.daemon.register(obj)
@@ -610,7 +654,7 @@ class ServerTestsOnce(unittest.TestCase):
                 self.daemon.unregister(obj)
                 result = p.echo(obj)
                 self.assertIsInstance(result, ServerTestObject, "serialized object must still be normal object")
-                Pyro4.config.AUTOPROXY = True  # make sure autoproxying is enabled
+                config.AUTOPROXY = True  # make sure autoproxying is enabled
                 result = p.echo(obj)
                 self.assertIsInstance(result, ServerTestObject, "non-pyro object must be returned as normal class")
                 self.daemon.register(obj)
@@ -622,8 +666,8 @@ class ServerTestsOnce(unittest.TestCase):
                 # note: the custom serializer may still be active but it should be smart enough to see
                 # that the object is no longer a pyro object, and therefore, no proxy should be created.
         finally:
-            Pyro4.config.AUTOPROXY = True
-            Pyro4.config.SERIALIZER = "serpent"
+            config.AUTOPROXY = True
+            config.SERIALIZER = "serpent"
 
     def testConnectOnce(self):
         with Pyro4.core.Proxy(self.objectUri) as proxy:
@@ -631,11 +675,11 @@ class ServerTestsOnce(unittest.TestCase):
             self.assertFalse(proxy._pyroBind(), "second bind should not connect again")
 
     def testConnectingThreads(self):
-        class ConnectingThread(threadutil.Thread):
-            new_connections = threadutil.AtomicCounter()
+        class ConnectingThread(threading.Thread):
+            new_connections = AtomicCounter()
 
             def __init__(self, proxy, event):
-                threadutil.Thread.__init__(self)
+                threading.Thread.__init__(self)
                 self.proxy = proxy
                 self.event = event
                 self.setDaemon(True)
@@ -647,7 +691,7 @@ class ServerTestsOnce(unittest.TestCase):
                     ConnectingThread.new_connections.incr()  # 1 more new connection done
 
         with Pyro4.core.Proxy(self.objectUri) as proxy:
-            event = threadutil.Event()
+            event = threading.Event()
             threads = [ConnectingThread(proxy, event) for _ in range(20)]
             for t in threads:
                 t.start()
@@ -661,13 +705,13 @@ class ServerTestsOnce(unittest.TestCase):
             bigobject = [42] * 1000
             result = p.echo(bigobject)
             self.assertEqual(result, bigobject)
-            Pyro4.config.MAX_MESSAGE_SIZE = 999
+            config.MAX_MESSAGE_SIZE = 999
             try:
                 _ = p.echo(bigobject)
                 self.fail("should fail with ProtocolError msg too large")
             except Pyro4.errors.ProtocolError:
                 pass
-            Pyro4.config.MAX_MESSAGE_SIZE = 0
+            config.MAX_MESSAGE_SIZE = 0
 
     def testIterator(self):
         with Pyro4.core.Proxy(self.objectUri) as p:
@@ -687,6 +731,15 @@ class ServerTestsOnce(unittest.TestCase):
             self.assertEqual("one", next(generator))
             self.assertEqual("two", next(generator))
             self.assertEqual("three", next(generator))
+            self.assertEqual("four", next(generator))
+            self.assertEqual("five", next(generator))
+            with self.assertRaises(StopIteration):
+                next(generator)
+            with self.assertRaises(StopIteration):
+                next(generator)
+            generator.close()
+            generator = p.generator()
+            [v for v in generator]
             with self.assertRaises(StopIteration):
                 next(generator)
             generator.close()
@@ -707,17 +760,36 @@ class ServerTestsOnce(unittest.TestCase):
         p2._pyroRelease()
         p3._pyroRelease()
 
+    def testSerializedBlob(self):
+        sb = Pyro4.core.SerializedBlob("blobname", [1, 2, 3])
+        self.assertEqual("blobname", sb.info)
+        self.assertEqual([1, 2, 3],  sb.deserialized())
+
+    def testSerializedBlobMessage(self):
+        serializer = Pyro4.util.get_serializer("serpent")
+        data, _ = serializer.serializeCall("object", "method", ([1, 2, 3],), False)
+        msg = Pyro4.message.Message(Pyro4.message.MSG_INVOKE, data, serializer.serializer_id, 0, 42)
+        sb = Pyro4.core.SerializedBlob("blobname", msg, is_blob=True)
+        self.assertEqual("blobname", sb.info)
+        self.assertEqual(([1, 2, 3],), sb.deserialized())
+
+    def testProxySerializedBlobArg(self):
+        with Pyro4.core.Proxy(self.objectUri) as p:
+            blobinfo, blobdata = p.blob(Pyro4.core.SerializedBlob("blobname", [1, 2, 3]))
+            self.assertEqual("blobname", blobinfo)
+            self.assertEqual([1, 2, 3], blobdata)
+
 
 class ServerTestsThreadNoTimeout(unittest.TestCase):
     SERVERTYPE = "thread"
     COMMTIMEOUT = None
 
     def setUp(self):
-        Pyro4.config.LOGWIRE = True
-        Pyro4.config.POLLTIMEOUT = 0.1
-        Pyro4.config.SERVERTYPE = self.SERVERTYPE
-        Pyro4.config.COMMTIMEOUT = self.COMMTIMEOUT
-        Pyro4.config.SERIALIZERS_ACCEPTED.add("pickle")
+        config.LOGWIRE = True
+        config.POLLTIMEOUT = 0.1
+        config.SERVERTYPE = self.SERVERTYPE
+        config.COMMTIMEOUT = self.COMMTIMEOUT
+        config.SERIALIZERS_ACCEPTED.add("pickle")
         self.daemon = Pyro4.core.Daemon(port=0)
         obj = ServerTestObject()
         uri = self.daemon.register(obj, "something")
@@ -731,9 +803,9 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
         time.sleep(0.05)
         self.daemon.shutdown()
         self.daemonthread.join()
-        Pyro4.config.SERVERTYPE = "thread"
-        Pyro4.config.COMMTIMEOUT = None
-        Pyro4.config.SERIALIZERS_ACCEPTED.discard("pickle")
+        config.SERVERTYPE = "thread"
+        config.COMMTIMEOUT = None
+        config.SERIALIZERS_ACCEPTED.discard("pickle")
 
     def testConnectionStuff(self):
         p1 = Pyro4.core.Proxy(self.objectUri)
@@ -772,14 +844,14 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
         # test compression:
         try:
             with Pyro4.core.Proxy(self.objectUri) as p:
-                Pyro4.config.COMPRESSION = True
+                config.COMPRESSION = True
                 self.assertEqual(55, p.multiply(5, 11))
                 self.assertEqual("*" * 1000, p.multiply("*" * 500, 2))
         finally:
-            Pyro4.config.COMPRESSION = False
+            config.COMPRESSION = False
 
     def testOnewayMetaOn(self):
-        Pyro4.config.METADATA = True
+        config.METADATA = True
         with Pyro4.core.Proxy(self.objectUri) as p:
             self.assertEqual(set(), p._pyroOneway)  # when not bound, no meta info exchange has been done
             p._pyroBind()
@@ -798,7 +870,7 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
                 p.nonexisting_method()
 
     def testOnewayMetaOff(self):
-        Pyro4.config.METADATA = False
+        config.METADATA = False
         with Pyro4.core.Proxy(self.objectUri) as p:
             self.assertEqual(set(), p._pyroOneway)  # when not bound, no meta info exchange has been done
             p._pyroBind()
@@ -811,10 +883,10 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
             p._pyroOneway.add("nonexisting_method")
             # now it shouldn't fail because of oneway semantics (!) (and becaue there's no metadata to tell Pyro that the method doesn't exist)
             p.nonexisting_method()
-        Pyro4.config.METADATA = True
+        config.METADATA = True
 
     def testOnewayWithProxySubclass(self):
-        Pyro4.config.METADATA = False
+        config.METADATA = False
 
         class ProxyWithOneway(Pyro4.core.Proxy):
             def __init__(self, arg):
@@ -827,13 +899,13 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
             p._pyroOneway = set()
             self.assertEqual(55, p.oneway_multiply(5, 11))
             self.assertEqual(55, p.multiply(5, 11))
-        Pyro4.config.METADATA = True
+        config.METADATA = True
 
     def testOnewayDelayed(self):
         try:
             with Pyro4.core.Proxy(self.objectUri) as p:
                 p.ping()
-                Pyro4.config.ONEWAY_THREADED = True  # the default
+                config.ONEWAY_THREADED = True  # the default
                 now = time.time()
                 p.oneway_delay(1)  # oneway so we should continue right away
                 time.sleep(0.01)
@@ -843,7 +915,7 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
                 self.assertTrue(time.time() - now < 0.2, "delay should be running in its own thread")
                 # make oneway calls run in the server thread
                 # we can change the config here and the server will pick it up on the fly
-                Pyro4.config.ONEWAY_THREADED = False
+                config.ONEWAY_THREADED = False
                 now = time.time()
                 p.oneway_delay(1)  # oneway so we should continue right away
                 time.sleep(0.01)
@@ -852,11 +924,11 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
                 self.assertEqual(55, p.multiply(5, 11), "expected a normal result from a non-oneway call")
                 self.assertFalse(time.time() - now < 0.2, "delay should be running in the server thread")
         finally:
-            Pyro4.config.ONEWAY_THREADED = True  # back to normal
+            config.ONEWAY_THREADED = True  # back to normal
 
     def testSerializeConnected(self):
         # online serialization tests
-        ser = Pyro4.util.get_serializer(Pyro4.config.SERIALIZER)
+        ser = Pyro4.util.get_serializer(config.SERIALIZER)
         proxy = Pyro4.core.Proxy(self.objectUri)
         proxy._pyroBind()
         self.assertIsNotNone(proxy._pyroConnection)
@@ -898,7 +970,7 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
                 del tb
 
     def testTimeoutCall(self):
-        Pyro4.config.COMMTIMEOUT = None
+        config.COMMTIMEOUT = None
         with Pyro4.core.Proxy(self.objectUri) as p:
             p.ping()
             start = time.time()
@@ -930,7 +1002,7 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
             self.assertEqual("receiving: timeout", str(e.exception))
 
     def testProxySharing(self):
-        class SharedProxyThread(threadutil.Thread):
+        class SharedProxyThread(threading.Thread):
             def __init__(self, proxy):
                 super(SharedProxyThread, self).__init__()
                 self.proxy = proxy
@@ -976,7 +1048,7 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
                 p._pyroRelease()
 
     def testServerParallelism(self):
-        class ClientThread(threadutil.Thread):
+        class ClientThread(threading.Thread):
             def __init__(self, uri, name):
                 super(ClientThread, self).__init__()
                 self.setDaemon(True)
@@ -1013,7 +1085,7 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
             self.assertFalse(t.error, "all threads should report no errors")
         del threads
         duration = time.time() - start
-        if Pyro4.config.SERVERTYPE == "multiplex":
+        if config.SERVERTYPE == "multiplex":
             # multiplex based server doesn't execute calls in parallel,
             # so 6 threads times 0.5 seconds =~ 3 seconds
             self.assertTrue(2.5 < duration < 3.5)
@@ -1030,13 +1102,13 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
             next(generator)
 
     def testGeneratorLinger(self):
-        orig_linger = Pyro4.config.ITER_STREAM_LINGER
-        orig_commt = Pyro4.config.COMMTIMEOUT
-        orig_pollt = Pyro4.config.POLLTIMEOUT
+        orig_linger = config.ITER_STREAM_LINGER
+        orig_commt = config.COMMTIMEOUT
+        orig_pollt = config.POLLTIMEOUT
         try:
-            Pyro4.config.ITER_STREAM_LINGER = 0.5
-            Pyro4.config.COMMTIMEOUT = 0.2
-            Pyro4.config.POLLTIMEOUT = 0.2
+            config.ITER_STREAM_LINGER = 0.5
+            config.COMMTIMEOUT = 0.2
+            config.POLLTIMEOUT = 0.2
             p = Pyro4.core.Proxy(self.objectUri)
             generator = p.generator()
             self.assertEqual("one", next(generator))
@@ -1052,18 +1124,19 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
             with self.assertRaises(Pyro4.errors.PyroError):  # should not be resumable anymore
                 next(generator)
         finally:
-            Pyro4.config.ITER_STREAM_LINGER = orig_linger
-            Pyro4.config.COMMTIMEOUT = orig_commt
-            Pyro4.config.POLLTIMEOUT = orig_pollt
+            config.ITER_STREAM_LINGER = orig_linger
+            config.COMMTIMEOUT = orig_commt
+            config.POLLTIMEOUT = orig_pollt
 
     def testGeneratorNoLinger(self):
-        orig_linger = Pyro4.config.ITER_STREAM_LINGER
+        orig_linger = config.ITER_STREAM_LINGER
         try:
             p = Pyro4.core.Proxy(self.objectUri)
-            Pyro4.config.ITER_STREAM_LINGER = 0  # disable linger
+            config.ITER_STREAM_LINGER = 0  # disable linger
             generator = p.generator()
             self.assertEqual("one", next(generator))
             p._pyroRelease()
+            time.sleep(0.2)
             with self.assertRaises(Pyro4.errors.ConnectionClosedError):
                 next(generator)
             p._pyroReconnect()
@@ -1071,7 +1144,7 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
                 next(generator)
             generator.close()
         finally:
-            Pyro4.config.ITER_STREAM_LINGER = orig_linger
+            config.ITER_STREAM_LINGER = orig_linger
 
 
 class ServerTestsMultiplexNoTimeout(ServerTestsThreadNoTimeout):
