@@ -4,20 +4,22 @@ Tests for the low level socket functions.
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
+from __future__ import print_function
 import socket
 import os
 import sys
 import platform
+import threading
 import time
 import unittest
 import Pyro4.socketutil as SU
-from Pyro4 import threadutil, errors
+import Pyro4.util
+import Pyro4.constants
+from Pyro4.configuration import config
+from Pyro4 import errors
 from Pyro4.socketserver.multiplexserver import SocketServer_Multiplex
 from Pyro4.socketserver.threadpoolserver import SocketServer_Threadpool
 from Pyro4.core import Daemon
-import Pyro4.message
-import Pyro4.util
-import Pyro4.constants
 from testsupport import *
 
 
@@ -46,10 +48,10 @@ class TestSocketStuff(unittest.TestCase):
 
 class TestSocketutil(unittest.TestCase):
     def setUp(self):
-        Pyro4.config.POLLTIMEOUT = 0.1
+        config.POLLTIMEOUT = 0.1
 
     def testGetIP(self):
-        Pyro4.config.PREFER_IP_VERSION = 4
+        config.PREFER_IP_VERSION = 4
         myip = SU.getIpAddress("")
         self.assertTrue(len(myip) > 4)
         myip = SU.getIpAddress("", workaround127=True)
@@ -65,31 +67,31 @@ class TestSocketutil(unittest.TestCase):
         self.assertIn(":", SU.getIpAddress("localhost", ipVersion=6))
 
     def testGetIpVersion4(self):
-        version = Pyro4.config.PREFER_IP_VERSION
+        version = config.PREFER_IP_VERSION
         try:
-            Pyro4.config.PREFER_IP_VERSION = 4
+            config.PREFER_IP_VERSION = 4
             self.assertEqual(4, SU.getIpVersion("127.0.0.1"))
             self.assertEqual(4, SU.getIpVersion("localhost"))
-            Pyro4.config.PREFER_IP_VERSION = 0
+            config.PREFER_IP_VERSION = 0
             self.assertEqual(4, SU.getIpVersion("127.0.0.1"))
         finally:
-            Pyro4.config.PREFER_IP_VERSION = version
+            config.PREFER_IP_VERSION = version
 
     @unittest.skipUnless(has_ipv6, "ipv6 testcase")
     def testGetIpVersion6(self):
-        version = Pyro4.config.PREFER_IP_VERSION
+        version = config.PREFER_IP_VERSION
         try:
-            Pyro4.config.PREFER_IP_VERSION = 6
+            config.PREFER_IP_VERSION = 6
             self.assertEqual(6, SU.getIpVersion("::1"))
             self.assertEqual(6, SU.getIpVersion("localhost"))
-            Pyro4.config.PREFER_IP_VERSION = 4
+            config.PREFER_IP_VERSION = 4
             self.assertEqual(4, SU.getIpVersion("127.0.0.1"))
             self.assertEqual(6, SU.getIpVersion("::1"))
-            Pyro4.config.PREFER_IP_VERSION = 0
+            config.PREFER_IP_VERSION = 0
             self.assertEqual(4, SU.getIpVersion("127.0.0.1"))
             self.assertEqual(6, SU.getIpVersion("::1"))
         finally:
-            Pyro4.config.PREFER_IP_VERSION = version
+            config.PREFER_IP_VERSION = version
 
     def testGetInterfaceAddress(self):
         self.assertTrue(SU.getInterfaceAddress("localhost").startswith("127."))
@@ -271,11 +273,10 @@ class TestSocketutil(unittest.TestCase):
         ss = SU.createBroadcastSocket((None, 0))
         port = ss.getsockname()[1]
         cs = SU.createBroadcastSocket()
-        for bcaddr in Pyro4.config.parseAddressesString(Pyro4.config.BROADCAST_ADDRS):
+        for bcaddr in config.parseAddressesString(config.BROADCAST_ADDRS):
             try:
                 cs.sendto(tobytes("monkey"), 0, (bcaddr, port))
-            except socket.error:
-                x = sys.exc_info()[1]
+            except socket.error as x:
                 err = getattr(x, "errno", x.args[0])
                 # handle some errno that some platforms like to throw
                 if err not in Pyro4.socketutil.ERRNO_EADDRNOTAVAIL and err not in Pyro4.socketutil.ERRNO_EADDRINUSE:
@@ -302,7 +303,7 @@ class TestSocketutil(unittest.TestCase):
         cs.close()
 
     def testMsgWaitallProblems2(self):
-        class ReceiveThread(threadutil.Thread):
+        class ReceiveThread(threading.Thread):
             def __init__(self, sock, sizes):
                 super(ReceiveThread, self).__init__()
                 self.sock = sock
@@ -334,13 +335,13 @@ class TestSocketutil(unittest.TestCase):
     def testMsgWaitAllConfig(self):
         if platform.system() == "Windows":
             # default config should be False on these platforms even though socket.MSG_WAITALL might exist
-            self.assertFalse(Pyro4.config.USE_MSG_WAITALL)
+            self.assertFalse(config.USE_MSG_WAITALL)
         else:
             # on all other platforms, default config should be True (as long as socket.MSG_WAITALL exists)
             if hasattr(socket, "MSG_WAITALL"):
-                self.assertTrue(Pyro4.config.USE_MSG_WAITALL)
+                self.assertTrue(config.USE_MSG_WAITALL)
             else:
-                self.assertFalse(Pyro4.config.USE_MSG_WAITALL)
+                self.assertFalse(config.USE_MSG_WAITALL)
 
 
 class ServerCallback(object):
@@ -410,23 +411,23 @@ class TestSocketServer(unittest.TestCase):
 
 class TestServerDOS_multiplex(unittest.TestCase):
     def setUp(self):
-        self.orig_poll_timeout = Pyro4.config.POLLTIMEOUT
-        self.orig_comm_timeout = Pyro4.config.COMMTIMEOUT
-        Pyro4.config.POLLTIMEOUT = 0.5
-        Pyro4.config.COMMTIMEOUT = 0.5
+        self.orig_poll_timeout = config.POLLTIMEOUT
+        self.orig_comm_timeout = config.COMMTIMEOUT
+        config.POLLTIMEOUT = 0.5
+        config.COMMTIMEOUT = 0.5
         self.socket_server = SocketServer_Multiplex
 
     def tearDown(self):
-        Pyro4.config.POLLTIMEOUT = self.orig_poll_timeout
-        Pyro4.config.COMMTIMEOUT = self.orig_comm_timeout
+        config.POLLTIMEOUT = self.orig_poll_timeout
+        config.COMMTIMEOUT = self.orig_comm_timeout
 
-    class ServerThread(threadutil.Thread):
+    class ServerThread(threading.Thread):
         def __init__(self, server, daemon):
-            threadutil.Thread.__init__(self)
+            threading.Thread.__init__(self)
             self.serv = server()
             self.serv.init(daemon(), "localhost", 0)
             self.locationStr = self.serv.locationStr
-            self.stop_loop = threadutil.Event()
+            self.stop_loop = threading.Event()
 
         def run(self):
             self.serv.loop(loopCondition=lambda: not self.stop_loop.is_set())
@@ -471,9 +472,9 @@ class TestServerDOS_multiplex(unittest.TestCase):
             csock = SU.createSocket(connect=(host, port))
             conn = SU.SocketConnection(csock, "uri")
             # send the handshake/connect data
-            ser = Pyro4.util.get_serializer_by_id(Pyro4.message.SERIALIZER_MARSHAL)
+            ser = Pyro4.util.get_serializer_by_id(Pyro4.util.MarshalSerializer.serializer_id)
             data, _ = ser.serializeData({"handshake": "hello", "object": Pyro4.constants.DAEMON_NAME}, False)
-            msg = Pyro4.message.Message(Pyro4.message.MSG_CONNECT, data, Pyro4.message.SERIALIZER_MARSHAL, 0, 0)
+            msg = Pyro4.message.Message(Pyro4.message.MSG_CONNECT, data, Pyro4.util.MarshalSerializer.serializer_id, 0, 0)
             conn.send(msg.to_bytes())
             # get the handshake/connect response
             Pyro4.message.Message.recv(conn, [Pyro4.message.MSG_CONNECTOK])
@@ -521,14 +522,14 @@ class TestServerDOS_threading(TestServerDOS_multiplex):
     def setUp(self):
         super(TestServerDOS_threading, self).setUp()
         self.socket_server = SocketServer_Threadpool
-        self.orig_numthreads = Pyro4.config.THREADPOOL_SIZE
-        self.orig_numthreads_min = Pyro4.config.THREADPOOL_SIZE_MIN
-        Pyro4.config.THREADPOOL_SIZE = 1
-        Pyro4.config.THREADPOOL_SIZE_MIN = 1
+        self.orig_numthreads = config.THREADPOOL_SIZE
+        self.orig_numthreads_min = config.THREADPOOL_SIZE_MIN
+        config.THREADPOOL_SIZE = 1
+        config.THREADPOOL_SIZE_MIN = 1
 
     def tearDown(self):
-        Pyro4.config.THREADPOOL_SIZE = self.orig_numthreads
-        Pyro4.config.THREADPOOL_SIZE_MIN = self.orig_numthreads_min
+        config.THREADPOOL_SIZE = self.orig_numthreads
+        config.THREADPOOL_SIZE_MIN = self.orig_numthreads_min
 
 
 if __name__ == "__main__":
