@@ -8,6 +8,7 @@ import sys
 import os
 import unittest
 import Pyro4.util
+from Pyro4.configuration import config
 from testsupport import *
 
 
@@ -70,8 +71,7 @@ class TestUtils(unittest.TestCase):
         try:
             crash("stringvalue")
             self.fail("must crash with TypeError")
-        except TypeError:
-            x = sys.exc_info()[1]
+        except TypeError as x:
             x._pyroTraceback = pyro_tb  # set the remote traceback info
             pyrotb = "".join(Pyro4.util.getPyroTraceback())
             self.assertIn("Remote traceback", pyrotb)
@@ -137,50 +137,50 @@ class TestUtils(unittest.TestCase):
             del os.environ["PYRO_NS_PORT"]
         if "PYRO_COMPRESSION" in os.environ:
             del os.environ["PYRO_COMPRESSION"]
-        Pyro4.config.reset()
+        config.reset()
 
     def testConfig(self):
         self.clearEnv()
         try:
-            self.assertEqual(9090, Pyro4.config.NS_PORT)
-            self.assertEqual("localhost", Pyro4.config.HOST)
-            self.assertEqual(False, Pyro4.config.COMPRESSION)
+            self.assertEqual(9090, config.NS_PORT)
+            self.assertEqual("localhost", config.HOST)
+            self.assertEqual(False, config.COMPRESSION)
             os.environ["NS_PORT"] = "4444"
-            Pyro4.config.reset()
-            self.assertEqual(9090, Pyro4.config.NS_PORT)
+            config.reset()
+            self.assertEqual(9090, config.NS_PORT)
             os.environ["PYRO_NS_PORT"] = "4444"
             os.environ["PYRO_HOST"] = "something.com"
             os.environ["PYRO_COMPRESSION"] = "OFF"
-            Pyro4.config.reset()
-            self.assertEqual(4444, Pyro4.config.NS_PORT)
-            self.assertEqual("something.com", Pyro4.config.HOST)
-            self.assertEqual(False, Pyro4.config.COMPRESSION)
+            config.reset()
+            self.assertEqual(4444, config.NS_PORT)
+            self.assertEqual("something.com", config.HOST)
+            self.assertEqual(False, config.COMPRESSION)
         finally:
             self.clearEnv()
-            self.assertEqual(9090, Pyro4.config.NS_PORT)
-            self.assertEqual("localhost", Pyro4.config.HOST)
-            self.assertEqual(False, Pyro4.config.COMPRESSION)
+            self.assertEqual(9090, config.NS_PORT)
+            self.assertEqual("localhost", config.HOST)
+            self.assertEqual(False, config.COMPRESSION)
 
     def testConfigReset(self):
         try:
-            Pyro4.config.reset()
-            self.assertEqual("localhost", Pyro4.config.HOST)
-            Pyro4.config.HOST = "foobar"
-            self.assertEqual("foobar", Pyro4.config.HOST)
-            Pyro4.config.reset()
-            self.assertEqual("localhost", Pyro4.config.HOST)
+            config.reset()
+            self.assertEqual("localhost", config.HOST)
+            config.HOST = "foobar"
+            self.assertEqual("foobar", config.HOST)
+            config.reset()
+            self.assertEqual("localhost", config.HOST)
             os.environ["PYRO_HOST"] = "foobar"
-            Pyro4.config.reset()
-            self.assertEqual("foobar", Pyro4.config.HOST)
+            config.reset()
+            self.assertEqual("foobar", config.HOST)
             del os.environ["PYRO_HOST"]
-            Pyro4.config.reset()
-            self.assertEqual("localhost", Pyro4.config.HOST)
+            config.reset()
+            self.assertEqual("localhost", config.HOST)
         finally:
             self.clearEnv()
 
     def testResolveAttr(self):
-        Pyro4.config.REQUIRE_EXPOSE = True
-        @Pyro4.expose
+        config.REQUIRE_EXPOSE = True
+        @Pyro4.core.expose
         class Exposed(object):
             def __init__(self, value):
                 self.propvalue = value
@@ -240,12 +240,12 @@ class TestUtils(unittest.TestCase):
         self.assertRaises(AttributeError, Pyro4.util.getAttribute, obj, "u.v")
         self.assertRaises(AttributeError, Pyro4.util.getAttribute, obj, "u.v.value")
         # try again but with require expose disabled
-        Pyro4.config.REQUIRE_EXPOSE = False
+        config.REQUIRE_EXPOSE = False
         self.assertIsInstance(Pyro4.util.getAttribute(obj, "u"), Unexposed)
         self.assertEqual("<a>", str(Pyro4.util.getAttribute(obj, "a")))
         self.assertRaises(AttributeError, Pyro4.util.getAttribute, obj, "u.v")  # still not allowed to follow the dots
         self.assertRaises(AttributeError, Pyro4.util.getAttribute, obj, "u.v.value")  # still not allowed to follow the dots
-        Pyro4.config.REQUIRE_EXPOSE = True
+        config.REQUIRE_EXPOSE = True
 
     def testUnicodeKwargs(self):
         # test the way the interpreter deals with unicode function kwargs
@@ -262,7 +262,7 @@ class TestUtils(unittest.TestCase):
 
 class TestMeta(unittest.TestCase):
     def setUp(self):
-        Pyro4.config.REQUIRE_EXPOSE = True
+        config.REQUIRE_EXPOSE = True
 
     def testBasic(self):
         o = MyThingFullExposed("irmen")
@@ -279,7 +279,7 @@ class TestMeta(unittest.TestCase):
         o = MyThingFullExposed("irmen")
         m1 = Pyro4.util.get_exposed_members(o)
         self.assertNotIn("newly_added_method", m1["methods"])
-        MyThingFullExposed.newly_added_method = Pyro4.expose(lambda self: None)
+        MyThingFullExposed.newly_added_method = Pyro4.core.expose(lambda self: None)
         m2 = Pyro4.util.get_exposed_members(o)
         self.assertNotIn("newly_added_method", m2["methods"])
         Pyro4.util.reset_exposed_members(o)
@@ -341,30 +341,30 @@ class TestMeta(unittest.TestCase):
     def testExposePrivateFails(self):
         with self.assertRaises(AttributeError):
             class Test1(object):
-                @Pyro4.expose
+                @Pyro4.core.expose
                 def _private(self):
                     pass
         with self.assertRaises(AttributeError):
             class Test3(object):
-                @Pyro4.expose
+                @Pyro4.core.expose
                 def __private(self):
                     pass
         with self.assertRaises(AttributeError):
-            @Pyro4.expose
+            @Pyro4.core.expose
             class _Test4(object):
                 pass
         with self.assertRaises(AttributeError):
-            @Pyro4.expose
+            @Pyro4.core.expose
             class __Test5(object):
                 pass
 
     def testExposeDunderOk(self):
         class Test1(object):
-            @Pyro4.expose
+            @Pyro4.core.expose
             def __dunder__(self):
                 pass
         self.assertTrue(Test1.__dunder__._pyroExposed)
-        @Pyro4.expose
+        @Pyro4.core.expose
         class Test2(object):
             def __dunder__(self):
                 pass
